#!/bin/bash

#install script that gets the majority of the nodewatch software installed.
#this should be run on the master node once the whole cluster is up and running

#alternatively(and recommended now), you can set up a master and node, cp /etc/nodes to
#/tmp/nodes, edit /tmp/nodes, removing all but the master and the first node.  Then prior
#to running this install script, export NODES=/tmp/nodes.  The install script will still
#reference /etc/nodes to configure things, but will use /tmp/nodes when installing things.
#This allows you to install mcms on a subset of the whole cluster, backup the first node,
#and then restore the image to all other nodes

#######################################################
######### defines of error codes / exit status ########
#######################################################
NORMAL=0
MISSINGSETTING=1
MISSINGTFTP=2
MISSINGBOOST=3
MISSINGAPACHEDEV=4
MISSINGFREETYPEDEV=5
MISSINGLIBART=6
MISSINGPNGDEVEL=7
MISSINGMISC=8
MISSINGPACKAGE=9
FAILEDBUILD=10
CANNOTINSTALLGANGLIA=100
SHOULDNOTHAPPEN=254

[ "x$NODES" == "x" ] && NODES=/etc/nodes
export NODES

service=`which service`

debian_check_and_install(){
 dpkg --list | grep -q $1
 if [ $? -ne 0 ] ; then
   echo You must have the $1 deb installed to continue. Attempting to install
   apt-get -q -y install $1 || ( echo Failed installing $1 ; exit $MISSINGPACKAGE )
 fi
}

suse_check_and_install(){
 rpm -qa | grep -q $1
 if [ $? -ne 0 ] ; then
   echo You must have the $1 rpm installed to continue. Attempting to install
   zypper install -y --auto-agree-with-licenses $1 || ( echo Failed installing $1 ; exit $MISSINGPACKAGE )
 fi
}

redhat_check_and_install(){
 rpm -qa | grep -q $1
 if [ $? -ne 0 ] ; then
   echo You must have the $1 rpm installed to continue. Attempting to install
   yum install -y $1 || ( echo Failed installing $1 ; exit $MISSINGPACKAGE )
 fi
}

systemd_reload(){
 [ -x /bin/systemctl ] && systemctl daemon-reload
}

##############################################
###### initial verification of options #######
##############################################

if [ ! -f config ] ; then
  echo "Config file missing!"
  exit
fi
. config

if [ "x$USES_NODEWATCH" == "x" ] ; then
  echo "You must set the USES_NODEWATCH variable in the config file"
  exit $MISSINGSETTING
fi

if [ "x$USES_IPMI" == "x" ] ; then
  echo "You must set the USES_IPMI variable in the config file"
  exit $MISSINGSETTING
fi
   
if [ "$USES_NODEWATCH" == "yes" -a "x$NODEWATCH_SERIAL" == "x" ] ; then
  echo "If using nodewatch, you must set the NODEWATCH_SERIAL variable in the config file"
  exit $MISSINGSETTING
fi

if [ "x$OS" == "x" ] ; then
  echo "You must set the OS variable in the config file"
  exit $MISSINGSETTING
fi

if [ "x$OSVERSION" == "x" ] ; then
  echo "You must set the OSVERSION variable in the config file"
  exit $MISSINGSETTING
fi

if [ "x$ARCH" == "x" ] ; then
  echo "You must set the ARCH variable in the config file"
  exit $MISSINGSETTING
fi 

if [ "x$MCMS_USER" == "x" ] ; then
  echo "You must set the MCMS_USER variable in the config file"
  exit $MISSINGSETTING
fi 

if [ "$USES_NODEWATCH" == "yes" -a "$USES_IPMI" == "yes" ] ; then
  echo "You can't enable both NODEWATCH and IPMI"
  exit $MISSINGSETTING
fi

if [ "x$NISDOM" == "x" ] ; then
  echo "You must set the NISDOM variable in the config file"
  exit $MISSINGSETTING
fi

if [ "x$NISMINGID" == "x" ] ; then
  echo "You must set the NISMINGID variable in the config file"
  exit $MISSINGSETTING
fi

###########prerequisite tests #############



export CHKLEVELS="345"
###### tftp server, openipmi, and ypserv/ypbind #######


case $OS in
 redhat)
   case $OSVERSION in
     fc17)
       redhat_check_and_install dhcp
       redhat_check_and_install ypserv
       redhat_check_and_install tftp-server
       redhat_check_and_install ypserv
       redhat_check_and_install ypbind
       redhat_check_and_install apr
       if [ "x$USES_IPMI" == "xyes" ] ; then
         redhat_check_and_install OpenIPMI-devel
       fi
       ;;
     rhel6)
       redhat_check_and_install dhcp
       redhat_check_and_install tftp-server
       if [ "x$USES_IPMI" == "xyes" ] ; then
         redhat_check_and_install OpenIPMI
         grep -q CentOS /etc/issue
         if [ $? -eq 0 ] ; then
           redhat_check_and_install OpenIPMI-devel
         else
           #redhat bug #704652 - OpenIPMI-devel package not available
           #without optional repo added
           #this one is built from the redhat source rpm by Microway
           rpm -U --replacepkgs rhel6/OpenIPMI-devel-2.0.16-14.el6.x86_64.rpm
         fi
       fi
       redhat_check_and_install ypserv
       redhat_check_and_install ypbind
       redhat_check_and_install apr
       ;;
     *)
       redhat_check_and_install tftp-server
       redhat_check_and_install ypserv
       redhat_check_and_install ypbind
       redhat_check_and_install apr
       if [ "x$USES_IPMI" == "xyes" ] ; then
         redhat_check_and_install OpenIPMI-devel
       fi
   esac
   ;;
 suse)
   suse_check_and_install tftp
   suse_check_and_install nfs-kernel-server
   suse_check_and_install dhcp-server
   suse_check_and_install ypserv
   suse_check_and_install ypbind
   suse_check_and_install popt-devel
   suse_check_and_install pam-devel

   systemd_reload
   if [ "x$USES_IPMI" == "xyes" ] ; then
     suse_check_and_install OpenIPMI-devel
   fi
   egrep -q "11.4|12.1|12.2|12.3" /etc/issue && export CHKLEVELS="35"
   ;;
 debian)
   debian_check_and_install tftpd-hpa
   debian_check_and_install nis
   if [ "x$USES_IPMI" == "xyes" ] ; then
     debian_check_and_install libopenipmi-dev
   fi
   ;;
 gentoo)
   if [ "`equery -q list tftp`" == "" ] ; then
     echo You must have the tftp package installed.  Attempting to install.
     emerge --quiet -n tftp || ( echo Failed installing tftp ; exit $MISSINGTFTP )
   fi
   if [ "`equery -q list ypserv`" == "" ] ; then
     echo You must have the ypserv package installed.  Attempting to install.
     emerge --quiet -n ypserv || ( echo Failed installing tftp ; exit $MISSINGPACKAGE )
   fi
   if [ "`equery -q list ypbind`" == "" ] ; then
     echo You must have the ypbind package installed.  Attempting to install.
     emerge --quiet -n ypbind || ( echo Failed installing tftp ; exit $MISSINGPACKAGE )
   fi
   ;;
 *)
   echo "Can't find OS method for verifying tftp installation"
   exit $SHOULDNOTHAPPEN
esac


#more prereq tests
case $OSVERSION in
 debian|ubuntu)
   debian_check_and_install librrd4
   debian_check_and_install dhcp3-server
   debian_check_and_install nfs-kernel-server
   debian_check_and_install libfreetype6-dev
   debian_check_and_install libpng12-dev
   debian_check_and_install libart-2.0-dev
   debian_check_and_install apache2
   debian_check_and_install libapache2-mod-php5
   ;;
 suse11.0|suse11.x|suse12.x)
   egrep -q "11.3|11.4|12.1|12.2|12.3" /etc/issue
   if [ $? -eq 0 ] ; then
     suse_check_and_install libpng14-14
   else
     suse_check_and_install libpng3
   fi
   suse_check_and_install apache2-mod_php5
   systemd_reload
   ;;
 fc9|fc10|fc11|fc17|rhel6)
   redhat_check_and_install mod_ssl
   redhat_check_and_install php-5
esac


########## make sure freetype2-devel is installed #################
case $OSVERSION in
 sles10|sles11|suse10.x|suse11.0|suse11.x|suse12.x)
   suse_check_and_install freetype2-devel
   suse_check_and_install libart_lgpl-devel
   #prior to updates for 12.1, all suse were trying to install libpng-devel, but I think it was not correct for 11.4 either, so changing anyway
   #rww 2012-6-5
   egrep -q "11.4|12.1|12.2|12.3" /etc/issue && suse_check_and_install libpng14-devel || suse_check_and_install libpng-devel 
   systemd_reload
   ;;
esac

######## gentoo warn about /usr/portage ##########
if [ $OS == "gentoo" ] ; then
  echo "This script expects your cluster to be using a shared /usr/portage."
  echo "That means that before running this script, you'll need to export"
  echo "/usr/portage from your master and nfs mount it on your nodes."
  echo
  echo "If this still needs to be done, you can ctrl-c now to exit.  If not, just"
  echo "hit enter to continue."
  read
fi

########## beta warnings ###########
case $OSVERSION in
 sles10|sles11|debian|ubuntu|gentoo)
   echo "Support of gentoo, debian, ubuntu, sles11, and sles10 is in beta"
   echo "stages. Once installed fully, things should all work, but some"
   echo "manual configuration may be necessary after the script completes to"
   echo "finish the installation."
   echo
   echo "This is normally installed by Microway personnel so this doesn't matter much.  If you are"
   echo "installing this yourself, you may need to reference the commands in the install script to"
   echo "see what the intended configuration is if anything fails."
   ;;
esac


######### validate chassis selection if using nodewatch #########
if [ "x$USES_NODEWATCH" = "xyes" ] ; then
  grep -q $MASTERCHASSIS knownchassis
  if [ $? -ne 0 ] ; then
    echo $MASTERCHASSIS not in the list of known chassis types.
    exit 5
  fi
  grep -q $NODECHASSIS knownchassis
  if [ $? -ne 0 ] ; then
    echo $NODECHASSIS not in the list of known chassis types.
    exit 5
  fi
  echo Since you are using nodewatch, you should also have the nodewatch converter
  echo box hooked into the first serial port on the master with a cable long 
  echo enough \(use extention\) to reach every nodewatch card.  Alternatively,
  echo you can hook up the cables in order, 1 at a time as you set the ids.
  echo 
  echo If you do not yet have the converter box, you can continue, but exit the nodewatch
  echo id flash utility at the end of this install.  You must manually set the ids
  echo later in that case.
fi

############## requires network-enable

echo Before running this script, the entire cluster or at least the master
echo and one node should be powered on with password-less logins
echo for root using the rcom, rcpf, etc. scripts.
echo "These scripts are installed by Microway's network-enable scripts."
echo "THIS RELIES ON PASSWORD-LESS RSH!"
echo
echo "You also need to have NAT working, allowing cluster nodes to access the outside network."

echo
echo Press enter to continue, or ctrl-C to quit.
read

##########################################################
###  lets get down to business and install some stuff ####
##########################################################

##################################
##### install rrdtool       ######
##################################

case $OSVERSION in
 gentoo)
   INSTALLRRD=false
   emerge --quiet -n rrdtool
   ;;
 debian|ubuntu)
   debian_check_and_install rrdtool
   INSTALLRRD=false
   ;;
 
 suse11.0|suse11.x|suse12.x)
   INSTALLRRD=false
   suse_check_and_install rrdtool
   ;;

 rhel6)
   INSTALLRRD=false
   redhat_check_and_install rrdtool
   ;;

 *)
   INSTALLRRD=true
   rpm -qa | grep -q rrdtool
   if [ $? -eq 0 ] ; then
     echo RRDtool provided by your distro.
     INSTALLRRD=false
   fi
   ;;
esac

##rrdtool is provided by most distros now, this code might not get used at all anymore
## RWW 2011-05-16

if [ $INSTALLRRD == "true" ] ; then
  echo installing rrdtool
  #remove a possibly conflicting rrdtool
  [ -d rrdtool-1.2.13 ] && rm -rf rrdtool-1.2.13

  tar xzf rrdtool-1.2.13.tar.gz
  cd rrdtool-1.2.13

  if [ $ARCH == "x86-64" ] ; then
     ./configure --host=i586 >& configure.log.mcms || ( echo Failed to configure rrdtool ; exit $FAILEDBUILD )
  else
     ./configure >& configure.log.mcms || ( echo Failed to configure rrdtool ; exit $FAILEDBUILD )
  fi

  make >& make.log.mcms
  make site-perl-install &> make-site-perl-install.log.mcms || ( echo Failed to make site-perl-install rrdtool ; exit $FAILEDBUILD )
  make install &> make-install.log.mcms || ( echo Failed to make install rrdtool ; exit $FAILEDBUILD )
  cd ..
  tar cf /tmp/rrdtool.tar -C /usr/local rrdtool-1.2.13
  rcpf-nodes /tmp/rrdtool.tar /tmp
  rcom-nodes-parallel tar xf /tmp/rrdtool.tar -C /usr/local

  GANGLIACFLAGS="-I/usr/local/rrdtool-1.2.13/include -L/usr/local/rrdtool-1.2.13/lib"
  GANGLIALDFLAGS="-L/usr/local/rrdtool-1.2.13/lib"
else
  GANGLIACFLAGS=""
  GANGLIALDFLAGS=""
  echo Using system provided rrdtool.
fi


##################################
##### install ganglia/gmond ######
##################################

echo Installing/configuring basic ganglia software


source_install_ganglia(){
  echo "Building gmond for your system"
  tar xzf ganglia-3.1.2.tar.gz -C /tmp
  dirsave=`pwd`
  cd /tmp/ganglia-3.1.2
  [ $ARCH == "x86-64" ] && uselibdir="--libdir=/usr/lib64"
  CFLAGS="$GANGLIACFLAGS" LDFLAGS="$GANGLIALDFLAGS" ./configure --with-gmetad $uselibdir >&configure.log
  make >&make.log
  echo "Gmond has been built.. now installing..."
  cd ..
  tar cf ganglia-3.1.2.built.tar ganglia-3.1.2
  rcpf ganglia-3.1.2.built.tar /tmp
  rcom-parallel "cd /tmp ; tar xf ganglia-3.1.2.built.tar 2>1 | grep -v time\ stamp ; cd ganglia-3.1.2 ; make install >&install.log ; cd .. ; rm ganglia-3.1.2.built.tar ; rm -rf ganglia-3.1.2"
  cd $dirsave
  rcpf etc/gmond.conf /etc/ganglia/
}


#checking that php has needed support

case $OSVERSION in
 gentoo)
   equery uses php | grep -v "^-" > /tmp/php-uses
   grep -q gd /tmp/php-uses && grep -q xml /tmp/php-uses && grep -q ctype /tmp/php-uses && grep -q pcre /tmp/php-uses 
   if [ $? -ne 0 ] ; then
     echo PHP was built without the gd extension.  Adding 
     echo dev-lang/php gd to /etc/portage/package.use and rebuilding.
     rcom "echo dev-lang/php gd >> /etc/portage/package.use"
     emerge --quiet -b php
     rcom-nodes emerge --quiet -k php
   fi
   ;;
 debian|ubuntu)
   debian_check_and_install php5-gd
   ;;
 fc9|fc10|fc11|fc17|rhel5|rhel6)
   redhat_check_and_install php-gd
   ;;
 suse10.x|suse11.0|sles10|sles11|suse12.x)
   suse_check_and_install php5-gd
   ;;
 suse11.x)
   suse_check_and_install php5-gd
   rpm -q php5 | grep -q 5.2.11-0.1 && ( echo "Suse 11.1's php 5.2.11-0.1 is broken.  Please downgrade to 5.2.9" ; exit $MISSINGPACKAGE )
   ;;
esac

## NFS related stuff ##

if [ $OSVERSION == "ubuntu" ] ; then
  rcom-nodes "export PATH=$PATH ; dpkg --list | grep -q nfs-common || apt-get -q -y install nfs-common"
fi

###### Add mounts and mountpoints for backup/restore ####

echo Setting up noderestore directory on master and nodes
mkdir -p /home/noderestore/backups/Slot{1,2,3,4,5,6}

rcom "[ -e /drive ] && (echo -ne /drive already exists on\  ; hostname ; echo not overwriting) || mkdir /drive"
grep /home/noderestore /etc/fstab | grep -q drive
if [ $? -ne 0 ] ; then
  echo "/home/noderestore    /drive   auto    bind   0 0" >> /etc/fstab
  mount /drive
fi

grep /home /etc/fstab | grep -q exports 
if [ $? -ne 0 ] ; then
  echo "/home                /exports/home          auto    bind   0 0" >> /etc/fstab
  mkdir -p /exports/home       
  mount /exports/home       
fi

rcom-nodes "grep -q $MASTER:/home/noderestore /etc/fstab || echo $MASTER:/home/noderestore	/drive	nfs4	rsize=32768,wsize=32768,proto=tcp,intr 0 0 >> /etc/fstab"

grep -q /home /etc/exports
if [ $? -eq 0 ] ; then
  grep /home /etc/exports | grep -q no_root_squash
  if [ $? -ne 0 ] ; then
    echo "***INVALID HOME EXPORT WITHOUT no_root_squash ***"
    echo "moving /etc/exports to /etc/exports-"`date --iso-8601=minutes`
    mv /etc/exports /etc/exports`date --iso-8601=minutes`
  fi
fi
grep -q /home /etc/exports
if [ $? -ne 0 ] ; then
  echo "Adding /home to /etc/exports"
  echo "/home           ${NETWORK}0/24(rw,no_root_squash,async,no_subtree_check)">>/etc/exports
fi

grep -q "^/exports" /etc/exports
if [ $? -ne 0 ] ; then
  echo "Adding /exports for nfs v4 to /etc/exports"
  echo "#nfs v4 section for mcms noderestore image" >> /etc/exports
  echo "/exports ${NETWORK}0/24(rw,fsid=0,no_root_squash,async,insecure,no_subtree_check)">>/etc/exports
  echo "/exports/home        ${NETWORK}0/24(rw,nohide,no_root_squash,async,insecure,no_subtree_check)">>/etc/exports
fi

############################################
echo Modifying slave script for proper master name.
cat $OSVERSION/slave.orig | sed s/insertmasterhere/$MASTER/ > slave
chmod 755 slave
echo Adding slave initscript on nodes.
rcpf slave /etc/init.d

suse-add-slave-dependencies() {
  for i in nfs ypbind
   do
    echo Setting $i to depend on slave script
    rcom-nodes "grep -q Required.*slave /etc/init.d/$i || sed -i -e \"s/\(.*equired.*tart.*\)/\1\ slave/\" /etc/init.d/$i"
  done
  echo Any other services that require the master to be up should also be
  echo set to depend on the slave script.  Check /etc/init.d/nfs on a node
  echo for an example.
}

case $OS in
 redhat)  
   rcom-nodes-parallel /sbin/chkconfig --level $CHKLEVELS slave on
   ;;
 suse)
   rcom-nodes-parallel /sbin/chkconfig slave $CHKLEVELS
   suse-add-slave-dependencies
   ;;
 debian)
   rcom-nodes-parallel "ln -sf ../init.d/slave /etc/rcS.d/S40slave"
   ;;
 gentoo)
   rcom-nodes-parallel /sbin/rc-update add slave default defaultX defaultx >&/dev/null
   ;;
 *)
   echo Could not enable the slave initscript for your OS choice.
esac

portmap=portmap

case $OSVERSION in
  fc9|fc10|fc11|fc17|suse11.x|suse12.x|sles11|rhel6|gentoo|ubuntu)
    portmap=rpcbind
    ;;
esac


########### enable nfs etc ##########
case $OS in
 suse)
   chkconfig $portmap $CHKLEVELS
   #newer suse eliminated the separate nfslock script
   [ -f /etc/init.d/nfslock ] && chkconfig nfslock $CHKLEVELS
   chkconfig nfsserver $CHKLEVELS
   rcom "PATH=\$PATH:/sbin:/usr/sbin /etc/init.d/$portmap start"
   /etc/init.d/nfsserver stop &>/dev/null
   [ -f /etc/init.d/nfslock ] && /etc/init.d/nfslock stop &>/dev/null
   [ -f /etc/init.d/nfslock ] && /etc/init.d/nfslock start
   /etc/init.d/nfsserver start
   [ -f /etc/init.d/nfslock ] && rcom-nodes "PATH=\$PATH:/sbin:/usr/sbin /etc/init.d/nfslock restart"
   rcom-nodes "PATH=\$PATH:/sbin:/usr/sbin /etc/init.d/nfs restart"
   rcom-nodes /sbin/chkconfig $portmap $CHKLEVELS
   [ -f /etc/init.d/nfslock ] && rcom-nodes /sbin/chkconfig nfslock $CHKLEVELS
   rcom-nodes /sbin/chkconfig nfs $CHKLEVELS
   ;; 
 redhat)
   chkconfig --level $CHKLEVELS $portmap on
   chkconfig --level $CHKLEVELS nfslock on
   chkconfig --level $CHKLEVELS nfs on
   if [ $OSVERSION == "fc17" ] ; then
     rcom $service $portmap start
     $service nfs-server stop >& /dev/null
     $service nfs-server start
     rcom-nodes $service netfs restart
     rcom-nodes /sbin/chkconfig --level $CHKLEVELS netfs on
   else
     rcom /etc/init.d/$portmap start
     /etc/init.d/nfs stop >&/dev/null 
     /etc/init.d/nfs start
     rcom-nodes /etc/init.d/netfs restart
     rcom-nodes /sbin/chkconfig --level $CHKLEVELS netfs on
   fi
   ;; 
 debian)
   echo "Debian already has $portmap enabled."
   /etc/init.d/$portmap start
   /etc/init.d/nfs-kernel-server restart
   rcom-nodes-parallel "grep -q \"\ \/drive\ nfs\" /proc/mounts || mount /drive"
   ;;
 gentoo)
   rc-update add $portmap default defaultX defaultx >&/dev/null
   rc-update add nfs default defaultX defaultx >&/dev/null
   rcom /etc/init.d/$portmap start
   /etc/init.d/nfs stop
   /etc/init.d/nfs start
   rcom-nodes-parallel "/sbin/rc-update add $portmap default defaultX defaultx >&/dev/null  ; /etc/init.d/nfsmount restart"
   ;;
 *)
   echo Unable to configure nfs server for your OS choice.
   exit $SHOULDNOTHAPPEN
   ;;
esac

################## /drive is available from this point on #####################################

#ganglia install

if [ $ARCH == "alpha" ] ; then
  source_install_ganglia
  echo Gmetad initscripts will need to be handled manually.
else
  case $OS in
   debian)
     #debian provides good ganglia packages in their standard repo
     apt-get -q -y install ganglia-monitor gmetad
     rcom-nodes PATH=$PATH apt-get -q -y install ganglia-monitor
    
     rcpf ${OSVERSION}/gmond.conf /etc/ganglia/
     rcpf debian/ganglia-monitor /etc/init.d/
     ;;

   redhat)
     if [ $ARCH == "x86-64" ] ; then
       case $OSVERSION in
        rhel6)
           rcpf "rhel6/ganglia-3.1.7-3.el6.x86_64.rpm rhel6/libconfuse-2.6-3.el6.x86_64.rpm rhel6/ganglia-gmond-3.1.7-3.el6.x86_64.rpm" /tmp
           rcom rpm -U --replacepkgs /tmp/ganglia-3.1.7-3.el6.x86_64.rpm /tmp/libconfuse-2.6-3.el6.x86_64.rpm /tmp/ganglia-gmond-3.1.7-3.el6.x86_64.rpm >&/dev/null
           rpm -U --replacepkgs rhel6/ganglia-gmetad-3.1.7-3.el6.x86_64.rpm 
           ;;
	rhel5)
           rcpf "rhel5/ganglia-gmond-3.1.2-1.x86_64.rpm rhel5/libconfuse-2.6-1.x86_64.rpm rhel5/libganglia-3_1_0-3.1.2-1.x86_64.rpm" /tmp
           rcom rpm -U --replacepkgs /tmp/ganglia-gmond-3.1.2-1.x86_64.rpm /tmp/libconfuse-2.6-1.x86_64.rpm /tmp/libganglia-3_1_0-3.1.2-1.x86_64.rpm >&/dev/null
           rpm -U --replacepkgs rhel5/ganglia-gmetad-3.1.2-1.x86_64.rpm
           ;;
         fc17)
           sed -i /etc/yum.conf -e s/keepcache=0/keepcache=1/
           yum install -y ganglia-gmond.x86_64 ganglia-gmetad.x86_64
           cd /var/cache/yum
           mkdir /tmp/ganglia-rpms
           find . | egrep "ganglia-3.*rpm|ganglia-gmon.*rpm|dejavu-lgc.*rpm|rrdtool.*rpm|libconfuse.*rpm" | while read gfiles ; do
             cp $gfiles /tmp/ganglia-rpms
           done
           cd -
           rcpf-nodes "-r /tmp/ganglia-rpms" /tmp
           rcom-nodes rpm -i --replacepkgs --replacefiles /tmp/ganglia-rpms/*
           ;;
         *)
           echo Only rhel6 64bit, rhel5 64bit and some Fedoras have had new ganglia packages generated.
           echo Cannot continue.
           exit
       esac
     else 
       echo Only rhel6, rhel5, and fc17 64bit have had new ganglia packages handled.  Older fedora will need manual help installing ganglia.
       echo Cannot continue.
       exit
     fi

     rcpf etc/gmond.conf /etc/ganglia
     rcpf initscripts/gmond.rh /etc/init.d/gmond
     ;;

   suse)    ##this 3.1.2 package has only been tested on 11.1. should be good on 11.0, but probably not 10.x
     #if [ $ARCH == "x86-64" -a $OSVERSION == "suse10.x" -o $OSVERSION == "suse11.0" -o $OSVERSION == "suse11.x" ] ; then
     if [ $ARCH == "x86-64" -a $OSVERSION == "suse11.0" -o $OSVERSION == "suse11.x" -o $OSVERSION == "suse12.x" ] ; then
       #ensure libconfuse0 is on both master and node
       zypper mr -kt
       zypper install -y --force libconfuse0
       confuserpm=`find /var/cache/zypp/packages/ | grep libconfuse0-[0-9] | head -n1`
       mkdir -p /drive/grpms
       cp $confuserpm /drive/grpms
       rcom-nodes rpm -i --replacepkgs --replacefiles /drive/grpms/`basename $confuserpm`
       #actually install it
       cd suse11.x
       cp libganglia-3_1_0-3.1.2-1.x86_64.rpm ganglia-gmond-3.1.2-1.x86_64.rpm /drive/grpms
       rcom rpm -U --replacepkgs /drive/grpms/ganglia-gmond-3.1.2-1.x86_64.rpm /drive/grpms/libganglia-3_1_0-3.1.2-1.x86_64.rpm
       rpm -U ganglia-gmetad-3.1.2-1.x86_64.rpm
       cd ..
     elif [ $ARCH == "x86-64" -a $OSVERSION == "sles11" ] ; then
       cd sles11
       mkdir -p /drive/grpms
       cp libganglia-3_1_0-3.1.2-1.x86_64.rpm ganglia-gmond-3.1.2-1.x86_64.rpm libconfuse-2.6-1suse11.x86_64.rpm  /drive/grpms/
       rcom rpm -U --replacepkgs /drive/grpms/libconfuse-2.6-1suse11.x86_64.rpm /drive/grpms/ganglia-gmond-3.1.2-1.x86_64.rpm /drive/grpms/libganglia-3_1_0-3.1.2-1.x86_64.rpm
       rpm -U ganglia-gmetad-3.1.2-1.x86_64.rpm
       cd ..
     else
       source_install_ganglia
       mkdir -p /var/lib/ganglia/rrds
       chown nobody /var/lib/ganglia/rrds
       cp etc/gmetad.conf /etc
       cp initscripts/gmetad.suse /etc/init.d/gmetad
     fi
     cat initscripts/gmond.suse.template | sed s/eth0/$ETHDEV/ > initscripts/gmond.suse
     rcpf initscripts/gmond.suse /etc/init.d/gmond
     rcpf etc/gmond.conf /etc/ganglia/
     ;;
   gentoo)
     emerge --quiet -n --onlydeps ganglia
     emerge --quiet -n -B ganglia
     quickpkg rrdtool
     rcom "emerge --quiet -n --usepkg ganglia"
     rcpf gentoo/gmond.conf /etc/ganglia/
     cp gentoo/gmetad.conf /etc/ganglia
     rcpf initscripts/gmond.gentoo /etc/init.d/gmond
     ;;
   *)
     echo "Failed to find appropriate OS method for installing ganglia."
     exit $SHOULDNOTHAPPEN
  esac
fi


##### restart gmond with initscript that was replaced earlier ####
case $OS in
 suse)
   rcom "chmod 755 /etc/init.d/gmond ; /sbin/chkconfig --del gmond ; /sbin/chkconfig gmond $CHKLEVELS"
   rcom "/etc/init.d/gmond restart" >&/dev/null
   chkconfig --del gmetad
   chkconfig --add gmetad
   ;;
 redhat)
   rcom "/sbin/service gmond restart >&/dev/null ; /sbin/chkconfig --level $CHKLEVELS gmond on"
   chkconfig --level $CHKLEVELS gmetad on
   ;;
 debian)
   rcom /etc/init.d/ganglia-monitor restart >&/dev/null
   ;;
 gentoo)
   rcom /sbin/rc-update add gmond default defaultX defaultx >&/dev/null
   rc-update add gmetad default defaultX defaultx >&/dev/null
   ;;
 *)
   echo "Could not install gmond initscript for your OS choice."
   exit $SHOULDNOTHAPPEN
esac


##########################################################
echo Distributing node backup scripts.
rcpf-nodes createbackup.sh /usr/local/sbin
rcpf-nodes createbackup-real.sh /usr/local/sbin
rcpf-nodes parted-static /usr/local/sbin
install preparerestore.sh /usr/local/sbin
install preparerestore-background.sh /usr/local/sbin


#########################################################
if [ $ARCH == "alpha" ] ; then
  echo "MCMS node backup/restore functionality is not available on alpha."
  echo "MCMS has not been tested on an alpha arch in a VERY long time."
else
  prevdir=`pwd`
  echo Creating tftpboot directory
  cp -a tftpboot /
  cd /tftpboot/initrd
  tar xfp dev.tar
  rm -f dev.tar
  mkdir /tftpboot/initrd/proc
  mkdir /tftpboot/initrd/var/run
  mkdir /tftpboot/initrd/tmp
  chmod 1777 /tftpboot/initrd/tmp
  find . | cpio --quiet -H newc -o | gzip -9 > /tftpboot/initrd.img.gz
  cd ..
  rm -rf initrd
  cd $prevdir
fi

########################################################
echo Enabling tftp server
case $OSVERSION in
 sles10|sles11|suse10.x|suse11.0|suse11.x|suse12.x)
   chkconfig tftp on
   sed -i /etc/xinetd.d/tftp -e s:srv/tftpboot:tftpboot:
   sed -i /etc/xinetd.d/tftp -e /disable/s/yes/no/i
   echo Restarting xinetd to enable tftp
   /etc/init.d/xinetd restart
   ;;
 rhel5|rhel6|fc9|fc10|fc11|fc17)
   chkconfig --level $CHKLEVELS tftp on
   sed -i /etc/xinetd.d/tftp -e s:var/lib/tftpboot:tftpboot:
   echo Restarting xinetd to enable tftp
   $service xinetd restart
   ;;
 debian|ubuntu)
   if grep -q '"/tftpboot"' /etc/default/tftpd-hpa ; then
     echo Ubuntu runs tftpd-hpa as a daemon instead of with inetd/xinetd.  /etc/default/tftpd-hpa has the correct export already.
   else
     echo Ubuntu runs tftpd-hpa as a daemon instead of with inetd/xinetd.  Setting the correct directory in /etc/default/tftpd-hpa and restarting it.
     sed -i /etc/default/tftpd-hpa -e 's:^[^#]*TFTP_DIRECTORY.*:TFTP_DIRECTORY="/tftpboot":'
     /etc/init.d/tftpd-hpa restart
   fi
   ;;
 gentoo)
   echo Creating tftp file in /etc/xinetd.d
   cp gentoo/tftp /etc/xinetd.d
   /etc/init.d/xinetd restart
   ;;
 *)
   echo "Could not find a way to enable the tftp server for your OSVERSION"
   exit $SHOULDNOTHAPPEN
esac

########################################################

echo Creating dhcpd.conf file.
macs=`rcom /sbin/ifconfig $ETHDEV | grep HWaddr | awk '{print $5}'`
echo $macs | ./GenDHCPconf.sh $NETWORK $NODESTARTIP $DOMAINNAME $OSVERSION

case $OS in
 gentoo)
   emerge --quiet -n dhcp
   mv /etc/dhcpd.conf /etc/dhcp/
   echo Modifying /etc/conf.d/dhcp to set interface.
   cat /etc/conf.d/dhcpd | sed "s/IFACE=.*/IFACE=$ETHDEV/" > /tmp/dhcp-new-conf
   mv /tmp/dhcp-new-conf /etc/conf.d/dhcpd
   rc-update add dhcp default defaultX defaultx >&/dev/null
   ;;
 suse)
   echo Replacing /etc/sysconfig/dhcpd to specify network to run on.
   [ -e /etc/sysconfig/dhcpd ] && \
   mv /etc/sysconfig/dhcpd /etc/sysconfig/dhcpd-`date --iso-8601=minutes`
   echo DHCPD_INTERFACE=$ETHDEV >/etc/sysconfig/dhcpd
   /etc/init.d/dhcpd start  
   chkconfig dhcpd $CHKLEVELS
   ;;
 redhat)
   echo Replacing /etc/sysconfig/dhcpd to specify network to run on.
   [ -e /etc/sysconfig/dhcpd ] && \
   mv /etc/sysconfig/dhcpd /etc/sysconfig/dhcpd-`date --iso-8601=minutes`
   echo DHCPDARGS=$ETHDEV >/etc/sysconfig/dhcpd
   [ $OSVERSION == "rhel6" ] && mv /etc/dhcpd.conf /etc/dhcp/
   $service dhcpd start
   chkconfig --level $CHKLEVELS dhcpd on
   ;;
 debian)
   echo "INTERFACES=$ETHDEV" > /etc/default/dhcp3-server
   mv /etc/dhcpd.conf /etc/dhcp3/
   echo /etc/init.d/dhcpd restart
   echo "Microway's debian already has dhcpd enabled"
esac  


##########################################################
echo Enabling nis accross the cluster
case $OS in
 debian)
   apt-get -q -y install nis
   if grep -q NISSERVER=true /etc/default/nis ; then
     true  #nis server already enabled
   else
     sed -i /etc/default/nis -e 's/^[^#]*NISSERVER.*/NISSERVER=true/'
   fi
   #get it actually working
   nisdomainname $NISDOM
   sed -i /var/yp/Makefile -e "s/^MINGID=.*/MINGID=$NISMINGID/"
   cd /var/yp ; make ; cd -
   if [ $OSVERSION == "ubuntu" ] ; then
      /etc/init.d/ypserv restart
      /etc/init.d/ypbind restart
   else
      /etc/init.d/nis restart
   fi
   
   #now install it on nodes
   rcom-nodes "PATH=$PATH apt-get -q -y install nis ; echo $NISDOM > /etc/defaultdomain ; nisdomainname $NISDOM"
   ;;
 suse)
   nisdomainname $NISDOM
   echo $NISDOM > /etc/defaultdomain
   rcom-nodes "echo $NISDOM > /etc/defaultdomain ; nisdomainname $NISDOM ; zypper install -y --auto-agree-with-licenses ypbind"
   ;;
 redhat)
   rcom "grep -q \"=$NISDOM\" /etc/sysconfig/network || echo \"NISDOMAIN=$NISDOM\" >> /etc/sysconfig/network"
   if [ $OSVERSION == "rhel6" ] ; then
     mkdir -p /drive/mcms-tmp
     cp rhel6/ypbind-1.20.4-29.el6.x86_64.rpm rhel6/yp-tools-2.9-10.el6.x86_64.rpm /drive/mcms-tmp
     rcom-nodes rpm -U --replacepkgs --replacefiles /drive/mcms-tmp/ypbind-1.20.4-29.el6.x86_64.rpm /drive/mcms-tmp/yp-tools-2.9-10.el6.x86_64.rpm
   else
     rcom-nodes yum install -y ypbind
   fi
   ;;
 gentoo)
   rcom "grep -q \"^nis_domain_eth0=$NISDOM\" /etc/conf.d/net || echo nis_domain_eth0=$NISDOM >> /etc/conf.d/net"
   quickpkg --include-config=y ypbind
   mkdir -p /drive/mcms-tmp
   cat > /drive/mcms-tmp/install-ypbind << EOYPBIND
if [ "`equery -q list ypbind`" == "" ] ; then
  emerge --quiet -n ypbind
fi
EOYPBIND
   chmod 755 /drive/mcms-tmp/install-ypbind
   rcom-nodes-parallel /drive/mcms-tmp/install-ypbind
   ;;
esac

case $OS in
 suse)
   rcom /sbin/chkconfig $portmap $CHKLEVELS
   if [ -f /etc/sysconfig/network/dhcp ] ; then
     if grep -q MODIFY_NIS_CONF.*yes /etc/sysconfig/network/dhcp ; then
       /etc/init.d/network stop
       sed -i -e "s/MODIFY_NIS_CONF=\"yes\"/MODIFY_NIS_CONF=\"no\"/" /etc/sysconfig/network/dhcp
       rcpf /etc/sysconfig/network/dhcp /etc/sysconfig/network/
       /etc/init.d/network start
     fi
   fi
   if [ -f /etc/sysconfig/network/config ] ; then
     if grep -q NETCONFIG_NIS_POLICY.*auto /etc/sysconfig/network/config ; then
       sed -i -e "s/NETCONFIG_NIS_POLICY=\"auto\"/NETCONFIG_NIS_POLICY=\"\"/" /etc/sysconfig/network/config
       rcpf /etc/sysconfig/network/config /etc/sysconfig/network/
     fi
   fi
   rcom /sbin/chkconfig ypbind $CHKLEVELS
   rcom mv /etc/yp.conf /etc/yp.conf-`date --iso-8601=minutes`
   rcom "echo ypserver $MASTER > /etc/yp.conf"
   if [ $OSVERSION == "suse12.x" ] ; then  #disable nscd to prevent nis passwords from not taking effect quickly
       rcom "/usr/bin/systemctl disable nscd.service ; /usr/bin/systemctl stop nscd.service"
   fi
   /sbin/chkconfig ypserv $CHKLEVELS
   /etc/init.d/ypserv start
   ;;
 redhat)
   rcom /sbin/chkconfig --level $CHKLEVELS $portmap on
   rcom /sbin/chkconfig --level 0126 ypbind off
   rcom /sbin/chkconfig --level $CHKLEVELS ypbind on
   rcom mv /etc/yp.conf /etc/yp.conf-`date --iso-8601=minutes`
   rcom "echo ypserver $MASTER > /etc/yp.conf"
   /sbin/chkconfig --level $CHKLEVELS ypserv on
   $service ypserv start
   ;;
 debian)
   if [ $OSVERSION == "ubuntu" ] ; then
     grep -q $MASTER /etc/yp.conf || echo ypserver $MASTER >> /etc/yp.conf
     /etc/init.d/ypserv restart
     rcom-nodes "export PATH=$PATH ; grep -q $MASTER /etc/yp.conf || echo ypserver $MASTER >> /etc/yp.conf ; /etc/init.d/ypbind restart"
   else
     rcom "export PATH=$PATH ; grep -q $MASTER /etc/yp.conf || echo ypserver $MASTER >> /etc/yp.conf ; /etc/init.d/nis restart"
   fi
   ;;
 gentoo)
   rcom-parallel "/sbin/rc-update add ypbind default defaultX defaultx >&/dev/null ; echo ypserver $MASTER > /etc/yp.conf"
   /sbin/rc-update add ypserv default defaultX defaultx >&/dev/null
   /etc/init.d/ypserv start
   ;;
 *)
   echo Could not configure nis clients for your OS choice.
esac

rcpf etc/nsswitch.conf /etc/nsswitch.temp.conf
rcom "mv /etc/nsswitch.conf /etc/nsswitch.conf.orig-`date --iso-8601=minutes`; mv /etc/nsswitch.temp.conf /etc/nsswitch.conf"

cd /var/yp
make >&/dev/null
cd - >&/dev/null

case $OS in
 redhat|suse|gentoo)
   rcom "PATH=\$PATH:/sbin:/usr/sbin $service ypbind start"
   ;;
 debian)
   if [ $OSVERSION == "ubuntu" ] ; then
     rcom-nodes "PATH=\$PATH:/sbin:/usr/sbin /etc/init.d/ypbind restart"
   else
     rcom-nodes "PATH=\$PATH:/sbin:/usr/sbin /etc/init.d/nis restart"
   fi
   ;;
esac


##########################################################
echo Installing web page


###### configuring web server #########
case $OS in
 redhat|suse)
   case $OSVERSION in
    suse9x|sles9)
      mv /etc/apache2 /etc/apache2.orig-`/bin/date --iso-8601=minutes`
      mv /etc/sysconfig/apache2 /etc/sysconfig/apache2-`/bin/date --iso-8601=minutes`
      cp suse9x/apache2-sysconfig /etc/sysconfig/apache2
      mv /etc/php.ini /etc/php.ini.orig-`date --iso-8601=minutes`
      cp $OSVERSION/php.ini /etc
      ;;
    suse10.x|sles10|sles11|suse11.0|suse11.x|suse12.x)
      mv /etc/sysconfig/apache2 /etc/sysconfig/apache2-`/bin/date --iso-8601=minutes`
      cp $OSVERSION/apache2-sysconfig /etc/sysconfig/apache2
      mv /etc/apache2 /etc/apache2.orig-`/bin/date --iso-8601=minutes`
      ;;
    fc17)
      #systemd makes a private tmp for apache, but this breaks mcms's usage of /tmp
      sed -i /lib/systemd/system/httpd.service -e s/PrivateTmp=true/PrivateTmp=false/
      mv /etc/httpd /etc/httpd.orig-`/bin/date --iso-8601=minutes`
      ;;
    *)
      mv /etc/httpd /etc/httpd.orig-`/bin/date --iso-8601=minutes`
      ;;
   esac
   ;;
 debian|ubuntu)
   rm -f /tmp/mcms-found-apache1 /tmp/mcms-found-apache2
   if [ -d /etc/apache ] ; then
     mv /etc/apache /etc/apache.orig-`/bin/date --iso-8601=minutes`
     touch /tmp/mcms-found-apache1
   elif [ -d /etc/apache2 ] ; then
     mv /etc/apache2 /etc/apache2.orig-`/bin/date --iso-8601=minutes`
     touch /tmp/mcms-found-apache2
   else
     echo Can not find apache configuration.
     exit $MISSINGMISC
   fi
   ;;
 gentoo)
   mv /etc/apache2 /etc/apache2.orig-`/bin/date --iso-8601=minutes`
   ;;
 *)
   echo Cannot configure apache on your OS choice
   exit $SHOULDNOTHAPPEN
esac

[ $ARCH == "x86" -a -f /etc/php.ini ] && sed -i /etc/php.ini -e "s/lib64/lib/"


#of course debian support had to complicate this...
if [ $OS == "debian" ] ; then
  if [ -f /tmp/mcms-found-apache1 ] ; then
    tar xjf $OSVERSION/$ARCH-httpd-etc.tbz -C /etc || \
    (echo "Error extracting $OSVERSION/$ARCH-httpd-etc.tbz" ; echo "Are you sure $OSVERSION is the correct version string?" ; exit)
    /etc/init.d/apache restart
  elif [ -f /tmp/mcms-found-apache2 ] ; then
    tar xjf $OSVERSION/$ARCH-httpd2-etc.tbz -C /etc || \
    (echo "Error extracting $OSVERSION/$ARCH-httpd2-etc.tbz" ; echo "Are you sure $OSVERSION is the correct version string?" ; exit)
    /etc/init.d/apache2 restart
  else
    echo You should not be able to get here. It should have failed earlier.
    exit $SHOULDNOTHAPPEN
  fi
else
  tar xjf $OSVERSION/$ARCH-httpd-etc.tbz -C /etc || \
  (echo "Error extracting $OSVERSION/$ARCH-httpd-etc.tbz" ; echo "Are you sure $OSVERSION is the correct version string?" ; exit)
fi


####### setting up sys admin privleges through mcms ########
if [ -f /etc/mcms.users ] ; then mv /etc/mcms.users /etc/mcms.users.orig-`date --iso-8601=minutes` ; fi
echo $MCMS_USER > /etc/mcms.users
cd src
gcc wrapper.c -lpam -o ../wrapper >& /dev/null || echo Could not build wrapper.  Using default build.  If you cannot log in to mcms, this is probably why.
cd ..
install -m4755 wrapper /usr/local/sbin

case $OS in
 suse)
   WWWDIR=/srv/www
   ;;
 redhat|debian|gentoo)
   WWWDIR=/var/www
   ;;
esac

cd $WWWDIR

case $OS in
 suse)
   mv htdocs htdocs.orig-`/bin/date --iso-8601=minutes`
   cd - >&/dev/null
   cp -a html $WWWDIR/htdocs
   mv $WWWDIR/htdocs/htpasswd $WWWDIR
   cd $WWWDIR
   chown -R wwwrun:nogroup htdocs
   chkconfig apache2 $CHKLEVELS
   WWWROOT=$WWWDIR/htdocs
   sed -i $WWWDIR/htdocs/ganglia/admin/.htaccess -e 's/\/var\/www\/htpasswd/\/srv\/www\/htpasswd/'
   ;;
 redhat)
   [ -d html ] && mv html html.orig-`/bin/date --iso-8601=minutes`
   mkdir html
   cd - >&/dev/null
   cp -a html $WWWDIR/
   cd $WWWDIR
   mv html/htpasswd .
   WWWROOT=$WWWDIR/html
   sed -i $WWWDIR/html/ganglia/admin/.htaccess -e 's/\/srv\/www\/htpasswd/\/var\/www\/htpasswd/'
   chown -R apache:apache html
   chkconfig --level $CHKLEVELS httpd on
   ;;
 debian)
   [ -d html ] && mv html html.orig-`/bin/date --iso-8601=minutes`
   mkdir html
   cd - >&/dev/null
   cp -a html $WWWDIR/
   cd $WWWDIR
   chown -R www-data:www-data html
   mv html/htpasswd .
   WWWROOT=$WWWDIR/html
   sed -i $WWWDIR/html/ganglia/admin/.htaccess -e 's/\/srv\/www\/htpasswd/\/var\/www\/htpasswd/'
   ;;
 gentoo)
   [ -d localhost/htdocs ] && mv localhost/htdocs localhost/htdocs.orig-`/bin/date --iso-8601=minutes`
   cd - >&/dev/null
   cp -a html $WWWDIR/localhost/htdocs
   mv $WWWDIR/localhost/htdocs/htpasswd $WWWDIR/
   chown -R apache:apache $WWWDIR/localhost/htdocs
   WWWROOT=$WWWDIR/htdocs
   sed -i $WWWDIR/htdocs/ganglia/admin/.htaccess -e 's/\/srv\/www\/htpasswd/\/var\/www\/htpasswd/'
   ;;
 *)
   echo Cannot install web documents and enable the server for your OS.
   exit $SHOULDNOTHAPPEN
esac

if [ $INSTALLRRD == "true" ] ; then
  echo "Modifying ganglia's conf.php to point to the correct rrdtool location"
  case $OS in
    suse)
      sed -i -e "s:usr/bin/rrdtool:usr/local/rrdtool-1.2.13/bin/rrdtool:" /srv/www/htdocs/ganglia/conf.php
      ;;
    gentoo)
      sed -i -e "s:usr/bin/rrdtool:usr/local/rrdtool-1.2.13/bin/rrdtool:" /var/www/localhost/htdocs/ganglia/conf.php
      ;;
    *)
      sed -i -e "s:usr/bin/rrdtool:usr/local/rrdtool-1.2.13/bin/rrdtool:" /var/www/html/ganglia/conf.php
      ;;
  esac
fi

cd - >&/dev/null

########################################
### NODEWATCH install and config #######
########################################
install_serial_console() {


######## build and install anyterm ####################
  [ -d anyterm-1.1.29 ] && rm -rf anyterm-1.1.29
  tar xjf anyterm-1.1.29.tbz
  cd anyterm-1.1.29
  killall anytermd >& /dev/null
  rm -f /usr/local/bin/anytermd
  #futex.h in rhel/centos 5.3 is broken
  if [ $OSVERSION == "rhel5" ] ; then
    cp /usr/include/linux/futex.h /usr/include/linux/futex.h.orig
    cp ../src/futex.h.works /usr/include/linux/futex.h
  fi
  make >&/dev/null && cp anytermd /usr/local/bin >& /dev/null || echo Failure installing anyterm
  if [ $OSVERSION == "rhel5" ] ; then
    mv /usr/include/linux/futex.h.orig /usr/include/linux/futex.h
  fi
  cd - >&/dev/null
  ldconfig


#per node setup
  [ "$USES_IPMI" == "yes" ] &&  ./enable-serial-console.sh #master needs it enabled too if ipmi
  cp enable-serial-console.sh /drive
  rcom-nodes /drive/enable-serial-console.sh
}
 
##### has nodewatch or ipmi #####
if [ $USES_NODEWATCH = "yes" -o $USES_IPMI = "yes" ] ; then
  echo installing nodewatch and ipmi utils
  if [ $OS == "suse" ] ; then
    sed -i src/nodecontrol.c -e "s/var\/www\/html/srv\/www\/htdocs/"
  else
    sed -i src/nodecontrol.c -e "s/srv\/www\/htdocs/var\/www\/html/"
  fi
  gcc src/nodecontrol.c -o nodecontrol
  install nodecontrol /usr/local/sbin
  case $OS in
   suse)
     cd src
     (echo "#define DATFILE \"/srv/www/htdocs/ganglia/data/nodewatch.dat\"" ; cat pullnwdata.c) >pullnwdata-compile.c
     gcc pullnwdata-compile.c -o ../pullnwdata
     cd ..
     ;;
   redhat|debian|gentoo)
     cd src
     (echo "#define DATFILE \"/var/www/html/ganglia/data/nodewatch.dat\"" ; cat pullnwdata.c) >pullnwdata-compile.c
     gcc pullnwdata-compile.c -o ../pullnwdata
     cd ..
     ;;
   *)
     echo Cannot build pullnwdata for your OS choice.
     exit $SHOULDNOTHAPPEN
  esac
  
  install pullnwdata /usr/local/sbin


### ganglia-reports for nodewatch use the old style where each node reports itself.  Under ipmi, the scripts are written to spoof the nodes automatically
#in in the reports from the master
######just nodewatch, not ipmi here#########
  if [ $USES_NODEWATCH == "yes" ] ; then

    ### config and install master's ganglia script for reporting nodewatch data ####
    cat nodewatch-ganglia-reports/ganglia-report-$MASTERCHASSIS.sh | \
      sed s/insertmasterhere/$MASTER/ > /usr/local/sbin/ganglia-report-$MASTERCHASSIS.sh
    ln -sf /usr/local/sbin/ganglia-report-$MASTERCHASSIS.sh /usr/local/sbin/ganglia-report.sh
  
    ### config and install node's ganglia script for reporting nodewatch data ####
    cat nodewatch-ganglia-reports/ganglia-report-$NODECHASSIS.sh | sed s/insertmasterhere/$MASTER/ > ganglia-report-$NODECHASSIS-updated.sh
    chmod 755 ganglia-report-$NODECHASSIS-updated.sh
    rcpf-nodes ganglia-report-$NODECHASSIS-updated.sh /usr/local/sbin/ganglia-report-$NODECHASSIS.sh
    rcom-nodes ln -sf /usr/local/sbin/ganglia-report-$NODECHASSIS.sh /usr/local/sbin/ganglia-report.sh
   
    rcpf $OSVERSION/crontab /etc

    case $OS in
     suse)
       echo running- ./gennodeconf.sh $MASTERCHASSIS $NODECHASSIS /etc/nodes $NODESTARTIP $NODEWATCH_SERIAL
       ./gennodeconf.sh $MASTERCHASSIS $NODECHASSIS /etc/nodes $NODESTARTIP $NODEWATCH_SERIAL > $WWWDIR/htdocs/ganglia/data/clusterconf.dat 
       rm -f $WWWDIR/htdocs/ganglia/data/cluster.dat $WWWDIR/htdocs/ganglia/data/chassis.dat
       ;;
     redhat|debian|gentoo)
       ./gennodeconf.sh $MASTERCHASSIS $NODECHASSIS /etc/nodes $NODESTARTIP $NODEWATCH_SERIAL > $WWWDIR/html/ganglia/data/clusterconf.dat
       rm -f $WWWDIR/html/ganglia/data/cluster.dat $WWWDIR/html/ganglia/data/chassis.dat
       ;;
     *)
       echo "Can't generate clusterconf.dat for your OS choice."
       exit $SHOULDNOTHAPPEN
    esac
     
    cd nodewatch-server/RELEASE09
    echo Cleaning nodewatch source directory.
    make clean >& /dev/null
    if [ $OS == "suse" ] ; then
      sed -i datafiles.h -e "s/var\/www\/html/srv\/www\/htdocs/"
    else
      sed -i datafiles.h -e "s/srv\/www\/htdocs/var\/www\/html/"
    fi
    echo Building nodewatch server.
    make >&/dev/null
    echo Installing nodewatch server.
    make install >&/dev/null
    cd ../..
   
    install_serial_console

    case $OS in
     suse)
       cp initscripts/nodewatch.suse /etc/init.d/nodewatch
       chkconfig nodewatch $CHKLEVELS
       sed -i -e "s/usesIPMI']=\"yes\"/usesIPMI']=\"no\"/" \
           $WWWDIR/htdocs/ganglia/admin/pagestuff.php
       sed -i -e "s/usesNodewatch']=\"no\"/usesNodewatch']=\"yes\"/" \
           $WWWDIR/htdocs/ganglia/admin/pagestuff.php
       if [ $NODEWATCH_SERIAL == "yes" ] ; then
         sed -i -e "s/usesSerialconsole']=\"no\"/usesSerialconsole']=\"yes\"/" \
           $WWWDIR/htdocs/ganglia/admin/pagestuff.php
       else
       sed -i -e "s/usesSerialconsole']=\"yes\"/usesSerialconsole']=\"no\"/" \
         $WWWDIR/htdocs/ganglia/admin/pagestuff.php
       fi
       ;;
     redhat)
       cp initscripts/nodewatch.rh /etc/init.d/nodewatch
       chkconfig --level $CHKLEVELS nodewatch on
       sed -i -e "s/usesIPMI']=\"yes\"/usesIPMI']=\"no\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       sed -i -e "s/usesNodewatch']=\"no\"/usesNodewatch']=\"yes\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       if [ $NODEWATCH_SERIAL == "yes" ] ; then
         sed -i -e "s/usesSerialconsole']=\"no\"/usesSerialconsole']=\"yes\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       else
         sed -i -e "s/usesSerialconsole']=\"yes\"/usesSerialconsole']=\"no\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       fi
       ;;
     gentoo)
       cp initscripts/nodewatch.gentoo /etc/init.d/nodewatch
       rc-update add nodewatch default defaultX defaultx >&/dev/null
       sed -i -e "s/usesIPMI']=\"yes\"/usesIPMI']=\"no\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       sed -i -e "s/usesNodewatch']=\"no\"/usesNodewatch']=\"yes\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       if [ $NODEWATCH_SERIAL == "yes" ] ; then
         sed -i -e "s/usesSerialconsole']=\"no\"/usesSerialconsole']=\"yes\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       else
         sed -i -e "s/usesSerialconsole']=\"yes\"/usesSerialconsole']=\"no\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       fi
       ;;
     debian)
       cp initscripts/nodewatch.deb /etc/init.d/nodewatch
       ln -s ../init.d/nodewatch /etc/rc2.d/S98nodewatch
       ln -s ../init.d/nodewatch /etc/rc3.d/S98nodewatch
       ln -s ../init.d/nodewatch /etc/rc4.d/S98nodewatch
       ln -s ../init.d/nodewatch /etc/rc5.d/S98nodewatch
       sed -i -e "s/usesIPMI']=\"yes\"/usesIPMI']=\"no\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       sed -i -e "s/usesNodewatch']=\"no\"/usesNodewatch']=\"yes\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       if [ $NODEWATCH_SERIAL == "yes" ] ; then
         sed -i -e "s/usesSerialconsole']=\"no\"/usesSerialconsole']=\"yes\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       else
         sed -i -e "s/usesSerialconsole']=\"yes\"/usesSerialconsole']=\"no\"/" \
           $WWWDIR/html/ganglia/admin/pagestuff.php
       fi
       ;;
     *)
       echo "Can't install the nodewatch initscript for you OS choice."
       exit $SHOULDNOTHAPPEN
    esac
  
    if [ $NODEWATCH_SERIAL == "yes" ] ; then
      case $OS in
        suse)
          cat uniqueserialconsole.sh.orig | \
            sed s:gdatadirtobereplaced:/srv/www/htdocs/ganglia/data: > uniqueserialconsole.sh
          wwwrun=wwwrun
          ;;
        *)
          wwwrun=apache
          cat uniqueserialconsole.sh.orig | \
            sed s:gdatadirtobereplaced:/var/www/html/ganglia/data: > uniqueserialconsole.sh
          ;;
      esac

      # add permissions to serial device for apache user
      serialgroup=`ls -l /dev/ttyS1 | awk '{print $4}'`
      grep -q ^$serialgroup.*$wwwrun /etc/group || \
        sed -i -e "s/\(^$serialgroup.*[^:]$\)/\1,$wwwrun/" \
               -e "s/\(^$serialgroup.*:$\)/\1$wwwrun/" /etc/group

      install uniqueserialconsole.sh /usr/local/sbin/
      install unique-minicom.sh /usr/local/bin
      cp etc/minirc.dfl /etc
   
    else
      case $OS in
       suse)
         cd $WWWDIR/htdocs/ganglia/admin/
         ;;
       *)
         cd $WWWDIR/html/ganglia/admin/
         ;;
      esac
      rm -rf serialconsole.php clearseriallock.php clearseriallockdumb.php localconsole serialconsole
      cd -
    fi

########Just IPMI, not nodewatch here
  elif [ $USES_IPMI = "yes" ] ; then
    cp ipmi-ganglia-report.pl /usr/local/sbin
    grep -q ipmi-ganglia-report /etc/crontab || \
      echo '*/10 *  *  *  *  /usr/local/sbin/ipmi-ganglia-report.pl' >> /etc/crontab
    
    case $OS in
     suse)
       ./genipminodeconf.sh $MASTERCHASSIS $NODECHASSIS /etc/nodes $IPMI_USER $IPMI_PASSWORD www > $WWWDIR/htdocs/ganglia/data/cluster.dat
       rm -f $WWWDIR/htdocs/ganglia/data/clusterconf.dat $WWWDIR/htdocs/ganglia/data/chassisconf.dat
       ;;
     debian)
       ./genipminodeconf.sh $MASTERCHASSIS $NODECHASSIS /etc/nodes $IPMI_USER $IPMI_PASSWORD www-data > $WWWDIR/html/ganglia/data/cluster.dat
       rm -f $WWWDIR/html/ganglia/data/clusterconf.dat $WWWDIR/html/ganglia/data/chassisconf.dat
       ;;
     redhat|debian|gentoo)
       ./genipminodeconf.sh $MASTERCHASSIS $NODECHASSIS /etc/nodes $IPMI_USER $IPMI_PASSWORD apache > $WWWDIR/html/ganglia/data/cluster.dat
       rm -f $WWWDIR/html/ganglia/data/clusterconf.dat $WWWDIR/html/ganglia/data/chassisconf.dat
       ;;
     *)
       echo "Can't generate cluster.dat for your OS choice."
       exit $SHOULDNOTHAPPEN
    esac
   
    cd ipmimon-$IPMIMON_VERSION
    if [ $OS == "suse" ] ; then
      sed -i defines.hh -e "s/var\/www\/html/srv\/www\/htdocs/"
    else
      sed -i defines.hh -e "s/srv\/www\/htdocs/var\/www\/html/"
    fi
    make >&/dev/null
    make install
    cd - >&/dev/null
 
    install_serial_console

    case $OS in
     suse)
       cp initscripts/ipmimon.suse /etc/init.d/ipmimon
       chkconfig ipmimon $CHKLEVELS
       sed -i -e "s/usesIPMI']=\"no\"/usesIPMI']=\"yes\"/" \
                 $WWWDIR/htdocs/ganglia/admin/pagestuff.php
       sed -i -e "s/usesNodewatch']=\"yes\"/usesNodewatch']=\"no\"/" \
                 $WWWDIR/htdocs/ganglia/admin/pagestuff.php
       sed -i -e "s/usesSerialconsole']=\"no\"/usesSerialconsole']=\"yes\"/" \
                   $WWWDIR/htdocs/ganglia/admin/pagestuff.php
       ;;
     redhat)
       cp initscripts/ipmimon.rh /etc/init.d/ipmimon
       chkconfig --level $CHKLEVELS ipmimon on
       sed -i -e "s/usesIPMI']=\"no\"/usesIPMI']=\"yes\"/" \
                 $WWWDIR/html/ganglia/admin/pagestuff.php
       sed -i -e "s/usesNodewatch']=\"yes\"/usesNodewatch']=\"no\"/" \
                 $WWWDIR/html/ganglia/admin/pagestuff.php
       sed -i -e "s/usesSerialconsole']=\"no\"/usesSerialconsole']=\"yes\"/" \
                 $WWWDIR/html/ganglia/admin/pagestuff.php
       ;;
     gentoo)
       cp initscripts/ipmimon.gentoo /etc/init.d/ipmimon
       rc-update add ipmimon default defaultX defaultx >&/dev/null
       sed -i -e "s/usesIPMI']=\"no\"/usesIPMI']=\"yes\"/" \
                 $WWWDIR/html/ganglia/admin/pagestuff.php
       sed -i -e "s/usesNodewatch']=\"yes\"/usesNodewatch']=\"no\"/" \
                 $WWWDIR/html/ganglia/admin/pagestuff.php
       sed -i -e "s/usesSerialconsole']=\"no\"/usesSerialconsole']=\"yes\"/" \
                 $WWWDIR/html/ganglia/admin/pagestuff.php
       ;;
     debian)
       cp initscripts/ipmimon.deb /etc/init.d/ipmimon
       ln -sf ../init.d/ipmimon /etc/rc3.d/S98ipmimon
       ln -sf ../init.d/ipmimon /etc/rc4.d/S98ipmimon
       ln -sf ../init.d/ipmimon /etc/rc5.d/S98ipmimon
       sed -i -e "s/usesIPMI']=\"no\"/usesIPMI']=\"yes\"/" \
                 $WWWDIR/html/ganglia/admin/pagestuff.php
       sed -i -e "s/usesNodewatch']=\"yes\"/usesNodewatch']=\"no\"/" \
                 $WWWDIR/html/ganglia/admin/pagestuff.php
       sed -i -e "s/usesSerialconsole']=\"no\"/usesSerialconsole']=\"yes\"/" \
                 $WWWDIR/html/ganglia/admin/pagestuff.php
       ;;
     *)
       echo "Can't install the ipmimon initscript for you OS choice."
       exit $SHOULDNOTHAPPEN
    esac
  fi
 
#### does not have nodewatch or IPMI ########
else
  case $OS in
   suse)
     sed -i -e "s/usesIPMI']=\"yes\"/usesIPMI']=\"no\"/" \
           /srv/www/htdocs/ganglia/admin/pagestuff.php
     sed -i -e "s/usesNodewatch']=\"yes\"/usesNodewatch']=\"no\"/" \
           /srv/www/htdocs/ganglia/admin/pagestuff.php
     sed -i -e "s/usesSerialconsole']=\"yes\"/usesSerialconsole']=\"no\"/" \
           /srv/www/htdocs/ganglia/admin/pagestuff.php
     ;;
   redhat|debian|gentoo)
     sed -i -e "s/usesIPMI']=\"yes\"/usesIPMI']=\"no\"/" \
           /var/www/html/ganglia/admin/pagestuff.php
     sed -i -e "s/usesNodewatch']=\"yes\"/usesNodewatch']=\"no\"/" \
           /var/www/html/ganglia/admin/pagestuff.php
     sed -i -e "s/usesSerialconsole']=\"yes\"/usesSerialconsole']=\"no\"/" \
           /var/www/html/ganglia/admin/pagestuff.php
     ;;
   *)
     echo "Can't toggle nodewatch off in the php code for you OS choice."
     exit $SHOULDNOTHAPPEN
  esac
fi


##########################################################
echo Starting services
/etc/init.d/gmetad restart
[ -f /etc/init.d/gmond ] && rcom /etc/init.d/gmond restart

case $OS in
 suse)
   systemd_reload
   /etc/init.d/apache2 start
   ;;
 debian)
   [ -f /etc/init.d/apache ] && /etc/init.d/apache start
   [ -f /etc/init.d/apache2 ] && /etc/init.d/apache2 start
   ;;
 redhat)
   $service httpd start
   ;;
 gentoo)
   /etc/init.d/apache2 start
   ;;
 *)
   echo "Can't start apache webserver for you OS choice."
   exit $SHOULDNOTHAPPEN
esac


##########################################################
echo Installing desktop icon
cd icon/
varhostname=`hostname`
mkdir -p /root/Desktop
mkdir -p /usr/share/icons
cp mcms-icon-64x64.png /usr/share/icons/
cp MCMS.desktop /root/Desktop/

MCMS_GROUP=`groups $MCMS_USER | awk '{print $3}'`
mkdir -p /home/$MCMS_USER/Desktop
cp MCMS.desktop /home/$MCMS_USER/Desktop/
chown -R $MCMS_USER:$MCMS_GROUP /home/$MCMS_USER/Desktop
cd ..

##########################################################
if [ $USES_NODEWATCH == "yes" ] ; then

  echo "Install Complete.  Look for errors in the above output.  Nodewatch service"
  echo "has been installed and enabled for a reboot, but has not been turned on"
  echo "yet.  nwserver -f will now be run to assign id's to the nodewatch cards"
  echo "Hook up the cable from the serial converter box to 1 node at a time."
  echo "You must move the cable in order throughout the cluster."
  echo "Starting location should be the last octet of the first ip and end at the"
  echo "last octet of the last ip."
  echo "----------------------------"
  #stop first in case this is a reinstall
  /etc/init.d/nodewatch stop >& /dev/null
  if [ $OS == "suse" ] ; then
    nwserver -p $WWWDIR/htdocs/ganglia/data -f
  else
    nwserver -f
  fi
  echo "OK.  ID's assigned. Starting nodewatch service now."
  /etc/init.d/nodewatch start
  echo "Good luck! :)"
  echo

##########################################################
elif [ $USES_IPMI == "yes" ] ; then
  echo "Install complete.  Look for errors in the above output."
  echo
  echo
  echo "You will likely need to configure the chassis for IPMI now."
  echo "To do that, look at the existing $WWWROOT/ganglia/data/chassis.dat file"
  echo "and run \"ipmitool sensor\" to see how to configure it."
  echo 
  echo "Once you have it configured, run \"/etc/init.d/ipmimon start\" to start the service."
else
  echo "Install complete.  Look for errors in the above output."
  echo "Good luck! :)"
fi

