#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pthread.h>

#include "cardconfig.h"

// Have to include in this order:
// cardconfig.h, net.h, exec.h, limits.h

#include "net.h"                // if you want to include exec.h, you have to
// include net.h
#include "exec.h"

// If you want to include limits, you first have to include above.
#include "limits.h"
#include "alert.h"
#include "syslogging.h"


/*  bug report Mike

CHANGES:

- add ability to print out the email errors with the -e option
- try to close the sock in all cases before returning.

I have found the following bugs or issues in the code,
will have to fix them:

0.  Spinning this fcn off as a thread would cause a memory leak
    and it would be difficult to stop it if it got stuck
   correction:  make it detached and asychronously cancelable.

1.  errors are not sent for every condition
 correction:  make an error handling function so the 
 info doesn't have to be copied to each line.

2.  sprintfs in error messages do not check if 
  they are over the 2000 character limits
  correction:  use snprintf, and avoid copying the body
  of the message in the first place

3.  the array for dealing with the addresses is an arbitrary limit.

correction:  read the addresses into the rcpt to: sections directly,
 without first storing as an array elsewhere.

4. the to line and the subject line should be exchanged.
  based on how email seems to look.

5. if a \n were to be embedded in the to or subject lines,
  this would cause subsequent headers to print in the body,
 and possibly errors with the mail server
correction:  ?

6  if the message body were to begin with non-spaces followed by a colon,
  as in 
warning: you have a warning
  this would cause a nonstandard header to be generated.
correction:  insert a blank line between the last header and the body
in the data.

7. both the email and shutdown versions of this should have within the thread,
   a while loop that tries 5 times to do whatever it is supposed to do,
   with incremental backoff.  the timeout in the mainloop should factor in
   the total time it would take to make the 5 attempts.


8.  what happens if the system 'runs out of threads'?  this type of error
   is theoretically not as serious as others; you could just wait a while
   and try again.  Is this case specifically handled?

9.  status_flag is a pointer to the data.  this allows the thread 
   to announce success and kill itself, with the data still available.
    but it doesn't have to be this way.  the DATA itself is a pointer
    to data allocated by the calling thread.

10. With the shutdown routine, consider using popen rather than system 
   to prevent stdout characters from accumulating with no place to go,
   and crashing the program.  Also, this would save any possible error
   messages, so they can be passed on to the user/administrator.

11.  The code has to be flexible regarding the behavior of mail servers.
ours for example will try to resolve the domain of sender addresses.
this could take some time.  this program tries to read a response immediately 
--- what is the timeout of that read?  Also, this code does not interpret
any error messages from the mail server, it just assumes that if it gets
text back then everything is OK.  the program could be tested ok, and then a 
year later when there is a new mail server then there is found a problem.

12.  worst of all, if there are any error messages (such as 'bad address')
then these messages are received and erased forever by this program,
making it very difficult for the end user to debug the cause
of a lack of email.  ANY RESPONSE FROM THE SERVER IS PRESUMED TO BE
AFFIRMATIVE.  THE CODE BELOW JUST GOES MERRILY ON ITS WAY.

13.  the system should send an email 5 minutes after the system starts
up (to make sure networking is completely up) and it should send an
email once a month.  optionally, these monthly emails should be sent
on specific days of the month, and to a Microway email address so we
can verify that the system is working.  If we (Microway) does not
receive a monthly email to support, then we should complain.  we can
check on each day who we are supposed to be receiving emails from.


14.  If there is an error, it never tries to close the
channel

15.  If the to: address does not have a server, it does not work.
The email server should be appended.


PTHREAD_THREADS_MAX is defined somewhere.  Where? - pthread.h

*/
extern int emailverbose;        // defined in host_l.c, and the result
// of evaluating the -e option, for Exposing Email Errors
// server responses get posted via info_msg.
void nodewatch_email ( struct nw_email *message )
{
  struct hostent *hostinfo;
  struct sockaddr_in name;
  int sock;
  char temp[2000], temp2[401];
  char dests[MAX_RECIPIENTS][MAX_ADDR_LENGTH];
  int count, count2 = 0, numdests = 0;

  // cancel settings changed by MJ:
  pthread_setcancelstate ( PTHREAD_CANCEL_ENABLE, NULL );
  pthread_setcanceltype ( PTHREAD_CANCEL_ASYNCHRONOUS, NULL );

  for ( count = 0; message->addresses[count] != '\0'; count++ ) {
    if ( message->addresses[count] == ',' ) {
      numdests++;
      count2 = 0;
    } else {
      dests[numdests][count2] = message->addresses[count];
      count2++;
    }
  }
  message->status_flag = ALERT_STATUS_BUSY;
  sock = socket ( PF_INET, SOCK_STREAM, IPPROTO_TCP );
  if ( sock < 0 ) {
    error_msg ( "email transmission socket error" );
    close ( sock );
    return;
  }
  name.sin_family = AF_INET;
  name.sin_port = htons ( 25 ); //mail server runs on port 25

//resolve hostname
  hostinfo = gethostbyname ( message->server );
  if ( hostinfo == NULL ) {
    hostinfo =
      gethostbyaddr ( message->server, strlen ( message->server ), AF_INET );
  }
  if ( hostinfo == NULL ) {
    sprintf ( temp,
              "Could not resolve %s.  The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, message->addresses, message->subject,
              message->body );
    error_msg ( temp );
    message->status_flag = ALERT_STATUS_ERROR;
    close ( sock );
    return;
  }
  memcpy ( &name.sin_addr, hostinfo->h_addr, hostinfo->h_length );

#ifdef EMAIL_DEBUG
  fprintf ( stderr, "ip=%d\n", hostinfo->h_addr );

#endif /*  */
//open socket      
  if ( connect ( sock, ( struct sockaddr * ) &name, sizeof ( name ) ) < 0 ) {
    sprintf ( temp,
              "Could not connect to %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    message->status_flag = ALERT_STATUS_ERROR;
    close ( sock );
    return;
  }
//////////////////////////////////////////////////   
//actually send message
//////////////////////////////////////////////
  //read in all the stuff a mail server pronounces about itself
  if ( read ( sock, temp, 1999 ) == 0 ) {
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
  if ( emailverbose )
    error_msg ( temp );

  //send the initial HELO
  sprintf ( temp, "HELO %s\n", message->server );
  if ( 0 == write ( sock, temp, strlen ( temp ) ) ) {   //verfity transmit
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
  if ( read ( sock, temp2, 400 ) == 0 ) {       //verify a response
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
  if ( emailverbose )
    error_msg ( temp2 );
  if ( 0 != strncmp ( temp2, "250", 3 ) ) {
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Error transmitting alert to %s (IP %s) -- HELO message failed\nThe following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
#ifdef EMAIL_DEBUG
  fprintf ( stderr, "Sent HELO\n" );

#endif /*  */
  //send the from:      
  sprintf ( temp, "mail from: %s\n", message->source_email );
  if ( 0 == write ( sock, temp, strlen ( temp ) ) ) {   //verfity transmit
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
  if ( read ( sock, temp2, 400 ) == 0 ) {       //verify a response
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
  if ( emailverbose )
    error_msg ( temp2 );
  if ( 0 != strncmp ( temp2, "250", 3 ) ) {
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Error transmitting alert to %s (IP %s) -- mail from: %s failed\nThe following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->source_email,
              message->addresses, message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
#ifdef EMAIL_DEBUG
  fprintf ( stderr, "Sent mail from:\n" );

#endif /*  */
  //send the to:
  for ( count = 0; count <= numdests; count++ ) {
    sprintf ( temp, "rcpt to: %s\n", dests[count] );
    if ( 0 == write ( sock, temp, strlen ( temp ) ) ) { //verfity transmit
      message->status_flag = ALERT_STATUS_ERROR;
      error_msg ( temp );
      sprintf ( temp,
                "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
                message->server, hostinfo->h_addr, message->addresses,
                message->subject, message->body );
      close ( sock );
      return;
    }
    if ( read ( sock, temp2, 400 ) == 0 ) {     //verify a response
      message->status_flag = ALERT_STATUS_ERROR;
      sprintf ( temp,
                "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
                message->server, hostinfo->h_addr, message->addresses,
                message->subject, message->body );
      close ( sock );
      return;
    }
    if ( emailverbose )
      error_msg ( temp2 );
    if ( 0 != strncmp ( temp2, "250", 3 ) ) {
      message->status_flag = ALERT_STATUS_ERROR;
      sprintf ( temp,
                "Error transmitting alert to %s (IP %s) -- rcpt to: %s failed\nThe following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
                message->server, hostinfo->h_addr, message->addresses,
                message->addresses, message->subject, message->body );
      error_msg ( temp );
      close ( sock );
      return;
    }
#ifdef EMAIL_DEBUG
    fprintf ( stderr, "Sent rcpt to:\n" );

#endif /*  */
  }

  //send DATA
  sprintf ( temp, "DATA\n" );
  if ( 0 == write ( sock, temp, strlen ( temp ) ) ) {   //verfity transmit
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
  if ( read ( sock, temp2, 400 ) == 0 ) {       //verify a response
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
  if ( emailverbose )
    error_msg ( temp2 );
  if ( 0 != strncmp ( temp2, "354", 3 ) ) {
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Error transmitting alert to %s (IP %s) -- DATA command failed\nThe following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
#ifdef EMAIL_DEBUG
  fprintf ( stderr, "Sent DATA\n" );

#endif /*  */

  //send the subject
  sprintf ( temp, "Subject: %s\n", message->subject );
  if ( 0 == write ( sock, temp, strlen ( temp ) ) ) {   //verfity transmit
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
#ifdef EMAIL_DEBUG
  fprintf ( stderr, "Sent Subj\n" );

#endif /*  */

  //send the To: line
  sprintf ( temp, "To: %s\n", message->addresses );
  if ( 0 == write ( sock, temp, strlen ( temp ) ) ) {   //verfity transmit
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
#ifdef EMAIL_DEBUG
  fprintf ( stderr, "Sent To:\n" );

#endif /*  */

  //send the body
  sprintf ( temp, "%s\n.\n", message->body );
  if ( 0 == write ( sock, temp, strlen ( temp ) ) ) {   //verify transmit
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    close ( sock );
    return;
  }
  if ( read ( sock, temp2, 400 ) == 0 ) {       //verify a response
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    close ( sock );
    return;
  }
  if ( emailverbose )
    error_msg ( temp2 );
  if ( 0 != strncmp ( temp2, "250", 3 ) ) {
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Error transmitting alert to %s (IP %s) -- BODY not accepted\nThe following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    error_msg ( temp );
    close ( sock );
    return;
  }
#ifdef EMAIL_DEBUG
  fprintf ( stderr, "Sent body\n" );

#endif /*  */

  //send the QUIT
  sprintf ( temp, "QUIT\n" );
  if ( 0 == write ( sock, temp, strlen ( temp ) ) ) {   //verfity transmit
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    close ( sock );
    return;
  }
#ifdef EMAIL_DEBUG
  fprintf ( stderr, "Sent QUIT\n" );

#endif /*  */

  //read in all the stuff a mail server pronounces about itself
  if ( read ( sock, temp, 1999 ) == 0 ) {
    message->status_flag = ALERT_STATUS_ERROR;
    sprintf ( temp,
              "Lost communication with %s (IP %s) The following alert could not be delivered:\nTo: %s\nSubject: %s\n%s\n",
              message->server, hostinfo->h_addr, message->addresses,
              message->subject, message->body );
    close ( sock );
    return;
  }
  if ( emailverbose )
    error_msg ( temp );

#ifdef EMAIL_DEBUG
  fprintf ( stderr, "Trying to close socket.\n" );

#endif /*  */
  close ( sock );
  message->status_flag = ALERT_STATUS_DONE;
  return;
}

void safeShutdown ( CLUSTERNODE * thenode )
{
  char temp[500];
  FILE *process;
  pt_dbug(__LINE__,"safeShutdown - about to generate rsh command: ",temp,NULL);
  // canceltype changed by MJ:
  pthread_setcancelstate ( PTHREAD_CANCEL_ENABLE, NULL );
  pthread_setcanceltype ( PTHREAD_CANCEL_ASYNCHRONOUS, NULL );
  thenode->os_shutdown_status_flag = OS_SHUTDOWN_BUSY;
  snprintf ( temp, 499, "rsh %s /sbin/halt &>/dev/null",
             thenode->ipaddress );
  pt_dbug(__LINE__,"safeShutdown - about to run rsh command: ",temp,NULL);
  //??? when system is used, what are the stdin and stdout 
  //??? of the process (rsh) called???
  //??? if they inherit the parent process, then popen should 
  //??? have been used instead, because any characters
  //??? generated by system would accumulate in a buffer
  //??? and eventually cause the program to crash!!!
  //???  
  //??? Not only that, but also can the stdout of the rsh process
  //??? be used to help determine success or failure, and if
  //??? failure, the reason for the failure???
  process=popen(temp,"r");
  if ( process==NULL ) {
    thenode->os_shutdown_status_flag = OS_SHUTDOWN_FAILED;
	 return;
  } else {
	 fgets(temp,500,process);
	 pclose(process);
	 pt_dbug(__LINE__,"popen output this- ",temp,NULL);
    thenode->os_shutdown_status_flag = OS_SHUTDOWN_IDLE;
    return;
  }
}
