
// datafiles.c

// manages data output to file, command input file,
//  config file, and range file.

#define RAWANALOGDATA 1
#define CALANALOGDATA 0
#define PRSCREEN 1
#define PRFILE 1

// Above define will force raw data to be printed, 
// accomodates transition to having the card do the math
// rather than the host.

// Note:  in this version, even if the card is doing the math,
// there is still the redundancy of analog channel assignment
// fixed separately in the card and in cardconfig.h



#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cardconfig.h"         // how the cards are physically set up.
#include "net.h"
#include "exec.h"
#include "codes.h"
#include "timer.h"
#include "host_l.h"
#include "limits.h"
#include "syslogging.h"
#include "datafiles.h"

#define DISPLAY_DELAY 2.0       // the time between updates. was 1.2, changed to 2
#define SCREEN_UPDATE_DELAY 0.5 // shorter time for screenupdate.
#define INIT_SCREEN_DELAY 4.0   // initial delay for valid data --was 3

extern int dumpmetrics;
extern int nodaemon;
//extern NODE Node[MAX_NODES];  // Mike changed to linked lists, below...
extern QUEUE QueueMessage;

// all from Limits.c:

extern CHASSIS *ChassisList;    // pointer to the start of the chassis 
// linked list database.

extern CLUSTERNODE *ClusterList;        // pointer to the start of the chassis
// linked list database.

extern CLUSTERNODE *NWMaster;   // pointer to this master (running
// this software) as stored in the chassis list.


extern unsigned int NodeCount;  // number of nodes in linked list
extern unsigned int ChassisCount;       // number of chasses defined
extern unsigned int TempavCount;        // number of nodes containing defined tempaverage probes.
// (if tempaverage = 0 for a node, that node is not included in tempaveragecount)


// All following defined in cardconfig.h. 
// these variables are set up here to allow
// runtime changes/inputs in the future:


double dVRef = CYGNAL_DVREF;    // voltage reference on the Cygnal chip.
// extern unsigned char cluster_nodes;     // This is set up in execnode.c

// static unsigned int thermo_array[] = THERMO_ARRAY;
// static unsigned int volt_array[] = VOLT_ARRAY;
// static unsigned int tack_array[] = TACK_ARRAY; // now defined within node chassis.
static unsigned int thermo_cygnal = THERMO_CYGNAL;
static unsigned int max_temps = MAX_TEMP;
//static unsigned int max_volts = MAX_VOLT;
//static double volt_r1_array[] = VOLT_R1_ARRAY;
//static double volt_r2_array[] = VOLT_R2_ARRAY;
//static double volt_nominal_array[] = VOLT_NOMINAL_ARRAY;
//static double volt_multiplier = VOLT_MULTIPLIER;
//static double thermo_multiplier = THERMO_MULTIPLIER;

// Thermistor calibration, found in cardconfig.h
static const double tema[] = TEMA_ARRAY;
static const double temb[] = TEMB_ARRAY;
static const double temc[] = TEMC_ARRAY;
static const double temd[] = TEMD_ARRAY;
static const double temsel[] = TEMSEL_ARRAY;

static long curtimeval;         // current time as reported by timer.c

//static int xl, yl;
static double dK1, dK2;

//static char *nodename = NULL;
//static char *outFileName = "/var/www/html/ganglia/data/nodewatch.dat";
extern char *outfilename;

FILE *outFile;

int file_data_open (  )
{

  char tmp_error[400];
  if ( nodaemon )               //we do not want these messages sent when running as daemon
    info_msg
      ( "Opening data file for overwrite output, locking file for write.\n" );
  //  outFile = stdout;

  // FIRST RENAME OLD FILE:
  /*
     filetime = mygetcurtime (  );
     sprintf(tmp_error,"%s_%d", outFileName,filetime);

     rename(outFileName, tmp_error);
   */
  // FOLLOWING LINE SETS FILE:  Comment out to use stdout


  outFile = fopen ( outfilename, "w" );
  if ( outFile == NULL ) {
    sprintf ( tmp_error, "ERROR: Unable to open output file %s\n",
              outfilename );
    error_msg ( tmp_error );
    return 1;                   //if file cannot be opened, exit
  }
  return ( 0 );                 // it was successful
  // Note: if file is locked, it will not block, it will return 1.
}

void file_data_close (  )
{
  if ( nodaemon )               //we do not want these messages sent when running as daemon              
    info_msg ( " Closing data file for output, unlocking the file.\n" );
  if ( outFile != stdout )
    fclose ( outFile );
}


void lineInitial ( void )
{
// Vtemp = 2.75 mV/C * TEMP + 775 mV
  dK1 = .775 * 2. / dVRef * 4096.;
  dK2 = ( 1. / ( .00275 * 2. / dVRef ) ) / 4096.;
  //   init_wholescreen();  // BRENDAN's function to start CURSES etc.
  setTimeout ( 5, INIT_SCREEN_DELAY );  // changed to 10, MLJ
  setTimeout ( 6, DISPLAY_DELAY );      // changed to 10, MLJ

}

static double voltsAtAnalogInput ( int adc, int gain )
{
  return ( ( ( double ) adc * dVRef / 4095. ) / ( double ) gain );
}

//static double sensorVolts ( double dVolts, double dR1, double dR2 )
//{
//  return ( ( dR1 + dR2 ) * dVolts / dR2 );
//}

#define R25 1.0E04              // thermistor resistance at 25 C
#define R   1.0E04              // series resistor

// Rt = R / (VRef / V - 1)
// where Rt is calculated thermistor resistance
//       R  is series divider resister
//       V  is divider voltage

static double Rt ( double dV )  // voltage divider?
{
  if ( dV <= 0. )
    return ( 0. );
  else if ( dV >= dVRef )
    //         return(DBL_MAX);
    return ( pow ( 2., 1024. ) );
  else
    return ( R / ( ( dVRef / dV ) - 1. ) );
}


// 1 / T = a + b(Ln Rt/R25) + c(Ln Rt/R25) + d(Ln Rt/R25)

static double T ( double r )
{
  int i;

  double ln1, ln2, ln3;

  if ( r >= temsel[0] )
    i = 0;
  else if ( r >= temsel[1] )
    i = 1;
  else if ( r >= temsel[2] )
    i = 2;
  else if ( r > temsel[3] )
    i = 3;
  else
    return ( 1. );

  ln1 = log ( r );              /// MLJ changed from log
  ln2 = ln1 * ln1;
  ln3 = ln2 * ln1;
  return ( 1. / ( tema[i] + temb[i] * ln1 + temc[i] * ln2 + temd[i] * ln3 ) );
}

double thermocoupleval ( int a2d )
{
  if ( a2d < SHORT_THERMO_LIMIT )
    return ( SHORT_THERMO_OUT );

  if ( a2d > UNPLUG_THERMO_LIMIT )
    return ( UNPLUG_THERMO_OUT );

  return ( T ( Rt ( voltsAtAnalogInput ( a2d, CYGNAL_ANALOG_GAIN ) ) / R25 ) -
           273.15 );
}


// the following packets are FROM the PC TO the CARD
// to REQUEST the corresponding information.
//PACKET ana = {2, U_NODE, ANA};
//PACKET tca = {2, U_NODE, TCA};
// above relocated to requestprocess in execnode.c

//static int bShowAlarm, bShowVersion, bShowSerial;

void myprintst ( char *mystrng )
{
  // prints string to both stdout and the file.
  if ( dumpmetrics )
    fprintf ( stderr, "%s", mystrng );

  fprintf ( outFile, "%s", mystrng );

}



void analogProcess (  )
{
  char npstr[256];              // scratchpad space to assemble outgoing string.
  int i;                        //, unitno, unitindex; // obsoleted by pointers.
  CLUSTERNODE *unitindex = ClusterList;
  CHASSIS *chassisindex;
  signed short int a2d;
  // nasty little bugger!  it was unsigned before.
  //command_update(); // Code in Brendan's system to read keystrokes.



  if ( isTimeout ( 6 ) )        // update analog historgram once each second
  {
    setTimeout ( 6, DISPLAY_DELAY );    // changed to 10, MLJ

    if ( file_data_open (  ) == 0 )     // set up data file output for the following data.
    {
      // Here is the loop through all the cards 
      // to print out the data in the right format:

      // First print preliminary information:
      curtimeval = mygetcurtime (  );
      /*   sprintf(npstr,
         "$version 1
         $time %x
         ", curtimeval);  */

      sprintf ( npstr, "$version 1\n" );
      myprintst ( ( char * ) npstr );
      // TIME HAS TO BE CHANGED TO HEX


      //      for ( unitindex = 0; unitindex < cluster_nodes; unitindex++ ) {
      while ( unitindex != NULL ) {
        // before even iding the node, give the node's last timestamp:
        chassisindex = unitindex->chassis;


        sprintf ( npstr, "$time %x\n",
                  ( unsigned int ) unitindex->nwdata.timestamp );
        myprintst ( npstr );

        // identify the node:

        sprintf ( npstr, "$NODE %u %s\n", unitindex->cardaddress,
                  unitindex->ipaddress );
        myprintst ( npstr );



        // Before anything, output if the card is alive:
        sprintf ( npstr, "$$DEAD  %u\n", unitindex->nwdata.alive );
        myprintst ( npstr );

        // Output card's version:
        sprintf ( npstr, "$$CARDVERSION  %u %u\n",
                  unitindex->nwdata.version.ucMajor,
                  unitindex->nwdata.version.ucMinor );
        myprintst ( npstr );

        //  First output  $$FAN and all the fans:
        sprintf ( npstr, "$$FAN  " );
        myprintst ( npstr );
        if ( chassisindex != NULL ) {
          for ( i = 0; i < MAX_TACK; i++ ) {
            sprintf ( npstr, "%05u ",
                      unitindex->nwdata.uaRpm[chassisindex->tackarray[i]] );
            myprintst ( npstr );

          }

          // Next output $$TEMP and all the temps
          sprintf ( npstr, "\n$$TEMP  " );
          myprintst ( npstr );

          //special case for Microway Infiniband switch
          if ( strcasestr(unitindex->chassis->name,"fastree")!=NULL ) {
            a2d = ( int ) unitindex->nwdata.uaA2D[thermo_cygnal];
#if RAWANALOGDATA

          sprintf ( npstr, "% 04d ", a2d );
          myprintst ( npstr );

#endif
#if CALANALOGDATA
          sprintf ( npstr, "% 4d  ",
                    ( int ) ( ( ( double ) a2d -
                                dK1 ) * dK2 * thermo_multiplier ) );
          myprintst ( npstr );

#endif
          } else {
            for ( i = 0; i < max_temps; i++ ) {
              // first indirect reference the correct analog channel:
              a2d = ( int ) unitindex->nwdata.uaA2D[chassisindex->thermoarray[i]];

#if RAWANALOGDATA

              sprintf ( npstr, "% 04d ", a2d );
              myprintst ( npstr );

#endif
#if CALANALOGDATA
              sprintf ( npstr, "% 4d  ",
                      ( int ) ( thermocoupleval ( a2d ) * thermo_multiplier ) );
              myprintst ( npstr );

#endif
            }

            // now output chip temperature:
            a2d = ( int ) unitindex->nwdata.uaA2D[thermo_cygnal];
#if RAWANALOGDATA

            sprintf ( npstr, "% 04d ", a2d );
            myprintst ( npstr );

#endif
#if CALANALOGDATA
            sprintf ( npstr, "% 4d  ",
                    ( int ) ( ( ( double ) a2d - dK1 ) * dK2 * thermo_multiplier ) );
            myprintst ( npstr );

#endif
          }
          // Then output $$VOLT and all the volts
          sprintf ( npstr, "\n$$VOLT  " );
          myprintst ( npstr );
          for ( i = 0; i < MAX_VOLT; i++ ) {
            // first indirect reference the correct analog channel:
            a2d = ( int ) unitindex->nwdata.uaA2D[chassisindex->voltarray[i]];
#if RAWANALOGDATA

            sprintf ( npstr, "% 04d ", a2d );
            myprintst ( npstr );
#endif
#if CALANALOGDATA
            sprintf ( npstr, "%4u  ",
                      ( int ) ( sensorVolts
                                ( voltsAtAnalogInput ( a2d, 1 ),
                                  volt_r1_array[i],
                                  volt_r2_array[i] ) * volt_multiplier ) );
            myprintst ( npstr );
#endif
          }

          // Then output $$POWER and the power status which for now
          // is derived from the voltage measurement.

          calcpowerstat ( unitindex );  // force an update to the 
          //                  power information; got to fix this later
          // to actually ask the card for this info instead...


          // Comment February 2003:  That may be true but
          // how would the card itself know about the power state?
          // It can know about the power _intent_ but not the 
          // power state. (as in, is it off because the 
          // power supply was turned off or because it is very broken?


          sprintf ( npstr, "\n$$POWER  %u",
                    ( int ) unitindex->nwdata.powerstat );
          myprintst ( npstr );
        }                       // end if chassisindex != NULL     
        // Then output $$VOLTNOMINAL and the nominal voltage later?

        sprintf ( npstr, "\n" );
        myprintst ( npstr );

        unitindex = unitindex->nextnode;        // go on to the next node

      }                         // end while loop
      file_data_close (  );
    }                           // end if filedata open
    //an error message could go here:  file could not be opened.
  }
}

void calcpowerstat ( CLUSTERNODE * unitindex )
{
  // temporary function to deduce the power supply on/off
  // state via the voltages.

  /*
     if ( unitindex->nwdata.uaA2D[(unitindex->chassis)->voltarray[0]] > 900 )       // if 12 volts is on
     unitindex->nwdata.powerstat = 1;
     else
     unitindex->nwdata.powerstat = 0;
   */

  // now it's done via power_supply_state
  unitindex->nwdata.powerstat = unitindex->power_supply_state;

}
