#include <stdio.h>
#include <stdio_ext.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "codes.h"
#include "net.h"                // for benefit of QUEUE declaration here:

#include "host_l.h"             // for callback function minimal_net()
#include "timer.h"
#include "flash.h"
extern int nodaemon;
extern unsigned char flash_override;
extern QUEUE QueueMessage;      // declared in netproce.c, this allows us to send
// out requests by enqueuing them for delivery.
extern unsigned char flash_override;    // toplevel state for flash_process(); set in host_l

// Note that whenever flash_override is non-zero, normal queries for measurements are suspended.

// The initial value of flash_override indicates what this program should do first:
// eventually, this will be controlled by a command line option to nodeserver.
//   0:  normal monitoring operation, no flash intervention.
//   1:  check values of poweroff limits at startup, then revert to normal operation.
//       (this is done b/c customer can change poweroff limits file; this should be
//        the default behavior for the released software.
//   2:  diagnosis mode:  continually monitor and print complete flash contents
//                        and other information for each card seen.
//   3:  manufacturing config mode:  store and verify serial num, fudge factors, etc.
//                  (causes leds to flash under host control to indicate success;
//                   eventually will also automatically test card and calculate fudge
//                   factors)
//   4.  integration config mode:    store node numbers, names etc.
//   5:  networking config mode (unusual) to change timeouts, baud rate, etc.
//       continually on all cards seen.
//   6:  networking limits test mode:  Commands the cards to take on new
//       networking parameters in RAM, for the purpose of stress-testing the network
//     (finding the fastest possible baud rate and shortest timeouts). 
//      This is changed in card RAM only
//     so when it fails, powercycling the cluster will return the cards to 
//     a known working state.
//   7:  possibly an integration test?  ie power all on and power all off?
//   8:  interactive flash debug.
//   9:  flash uniqueness test.  (stores and retrieves random values)
//  10:  flash persistence test (repeatedly retrieves values in random order)
//  11:  flash write protect test (optional)
char flash_state;               // main state of flash processor.  General state is also represented by flash_override.

/*
 This code manages a database of configuration information stored in the 8051 flash. 
 This code is called from execsens.c when incoming packets request or
 send over flash-based information.

 This code is also called upon reset to set up some pointers that point
 to flash-based configuration and operational parameters, such as the address.

In addition to critical parameters for the card, the flash is used to store 
lots of information whose meaning
only the master understands.  As a result, flash blocks are divided 
into fixed-sized segments, and the master can store anything it wants anywhere.
The same code and mechanism is used for arbitrary storage and card reconfiguration.

The master is responsible for reading data before writing to avoid unnecessary
writes to the flash, to read back the data after writing to check,
and to report errors in flash operation to the user.

*/
FL_WRREQ_PACKET fl_wrpak = {
  sizeof ( FL_WRREQ_PACKET ) - 1, 0, FLASH_WRITE
};


// Above reserves scratchpad space in memory to store a write request
// being assembled.

// Note that the initialized address is 0 and has to be repaired every time this is used!
FL_READREQ_PACKET fl_repak = {
  sizeof ( FL_READREQ_PACKET ) - 1, 0, FLASH_READ
};


// Above reserves scratchpad space to store a read request being assembled.
FL_WRREQ_PACKET fl_response = {
  sizeof ( FL_WRREQ_PACKET ) - 1, 0, FLASH_WRITE
};


// Above reserves space to store the most recently read flash packet.
// This data is a temporary holding area; if it is not consumed by the 
// time the next flash packet arrives, it is lost.

// Functions to drive flash management state machine and flash data/ackn receivers:

// Note that whenever flash_override is non-zero, normal queries for measurements are suspended.
// Note also that flash functions can BLOCK, disrupting normal loop in host_l.
void init_flashmachine (  )     // called by host at start.  sets consistent state; starts flash checks if necessary.
{
  if ( nodaemon )
    printf ( "Initializing card flash...\n" );
  if ( flash_override == FLASH_OVERRIDE_NOTSET )
    flash_override = DEFAULT_FLASH_OVERRIDE;
  flash_state = 0;
}

void flash_process (  )         // called every time by main host loop.  checks flash_override and flash_state.
// Now that we are using blocking functions, it's handy but less critical/complicated.
{
  switch ( flash_override ) {
     case 8:
       interactive_test (  );
       break;
     case 9:
       fl_uniqueness_test (  );
       break;
     case 10:
       fl_address_assign (  );
       break;
     default:
       break;                   // all is OK. do nothing.
  }
}


// Functions called by execnode to process incoming information:
void fl_copyresponse ( PACKET * inpacket )      // note incoming data or acknowledgement.
{
  int i;
  unsigned char *readpacket = ( unsigned char * ) inpacket;     // cast the pointer
  unsigned char *startstore = ( unsigned char * ) &fl_response; // cast the pointer
  if ( inpacket->ucLength != 4 && inpacket->ucLength != FL_SEGSIZE + 4 )
    printf ( "Flash packet length error\n" );
  for ( i = 0; i < ( inpacket->ucLength + 1 ); i++ )
    *( startstore + i ) = *( readpacket + i );
}

void fl_clearresponse (  )      // marks as invalid the response buffer
{
  fl_response.ucLength = FLRESP_CLRBYTE;
  fl_response.ucUnit = FLRESP_CLRBYTE;
  fl_response.ucCode = FLRESP_CLRBYTE;
  fl_response.block = FLRESP_CLRBYTE;
  fl_response.segment = FLRESP_CLRBYTE;
}

/////// Functions to deal with intermediate state machine states:

// NOTE:  the following functions BLOCK, and they call the callback function
// in host_l.c called minimal_net to keep things going.

// The reasoning behind blocking functions:
//  1.  we want each flash storage and read to take place IN ORDER, each with a +ackn.
//  2.  flash storages are not supposed to take a long time.  Disruption should be minimal.
//  3.  flash storages and reads happen infrequently, such as at startup and configuration.
//  4.  The callback functions to keep the network going during the block are simple and easy to implement.
//  5.  the logic and debugging of the flash procedures are clearer.  Programming is simpler and procedural.
//  6.  Despite the future attractiveness, I don't want multithreading at this time for ease/speed of debugging.

//  Lesson for the future:  if there are several parts of code
//  that are similar-looking, they should be combined into a single
// more general piece of code with individual callback fcns. for the differences.

/// individual segments:
int fl_request ( FL_WRREQ_PACKET * outpacket )  // queues up a generic request (could be read, write or erase) and sets up state machine to check for response
// blocks, int returns 1 for good or 0 for unsuccessful,
// with the response stored in fl_response in any case.
{

  // we assume the caller has already prepared the packet with the unit, block, segment, and data.
  // the caller is free to use globals such as 
  fl_clearresponse (  );        // recent data is invalidated.
  enqueue ( &QueueMessage, ( PACKET * ) outpacket, MAX_PRIORITY );      // queue up the request;
  // the caller already set up the outpacket when calling this function.
  setTimeout ( TIMER_FLASH, TIMEOUT_FLASH );
  while ( fl_response.ucLength == FLRESP_CLRBYTE
          && !( isTimeout ( TIMER_FLASH ) ) )
    minimal_net (  );
  minimal_net_purge (  );

  // at this point we either have timed out or we have a valid +ackn.
  // ... so see if we have a valid +ackn presumably from this write.  
  if ( fl_response.ucCode == outpacket->ucCode
       //&& fl_response.ucUnit == outpacket->ucUnit
       // removal of above line is a 
       // gross hack to ignore incoming address,
       // because they don't get updated fast enough.  
       && fl_response.block == outpacket->block
       && fl_response.segment == outpacket->segment )
    return ( 1 );               // success
  else
    return ( 0 );               // failure.
}


// The following interfaces deal with segments and blocks:
int fl_segread ( char unit, char block, char segment )
{
  fl_repak.ucUnit = unit;
  fl_repak.ucCode = FLASH_READ;
  fl_repak.block = block;
  fl_repak.segment = segment;
  return ( fl_request ( ( FL_WRREQ_PACKET * ) & fl_repak ) );
}

int fl_segwrite ( char unit, char block, char segment, unsigned char *ptseg )
{
//  int i;
  fl_wrpak.ucUnit = unit;
  fl_wrpak.ucCode = FLASH_WRITE;
  fl_wrpak.block = block;
  fl_wrpak.segment = segment;
  memcpy ( fl_wrpak.payload, ptseg, FL_SEGSIZE );       //replaces 2 lines below
//  for ( i = 0; i < FL_SEGSIZE; i++ )
//    fl_wrpak.payload[i] = *( ptseg + i );      // copy data into write area.
  return ( fl_request ( &fl_wrpak ) );
}

int fl_blkerase ( char unit, char block )
{
  fl_repak.ucUnit = unit;
  fl_repak.ucCode = FLASH_ERASE;
  fl_repak.block = block;
  fl_repak.segment = 0;
  return ( fl_request ( ( FL_WRREQ_PACKET * ) & fl_repak ) );
}

void interactive_test (  )
{

  // Here is where the fun begins... implement a simple user interface that reads, erases, writes, and rereads
  // blocks and segments of memory, printing the return sucssess/failures

  // I guess the user interface will be blocking... does it matter if the net is down waiting for user input?
  // not as long as it's for debugging or configuration purposes, and the queues are cleared before blocking.

  // sequence:
  // identify a block and segment, decide whether to erase it, determine the data, read, write and read again.
  // rinse, lather and repeat.
  int block, segment, unit, i, status;
  int datamode;                 // input how to generate data
  unsigned char data[FL_SEGSIZE];       // assembly of data itself.
  unsigned char theprintchar;
  printf ( "Welcome to interactive flash test.  We are blocking \
during user input; a network stoppage is normal.\n" );
  block = -1;
  segment = -1;
  unit = 1;
  datamode = 0;
  printf ( "Unit fixed at %d\n", unit );
  while ( block < 0 || block > FL_MAXBLOCK ) {
    printf ( "Enter the block number from 0 - 27\n" );
    scanf ( "%d", &block );
  }

  // input them as ints, then cast them to chars.
  // little endian plus less than 255, so int casts to char well.
  // hmmm.. I wonder how small valued ints cast to chars on big endian
  // machines... :)
  while ( segment < 0 || segment > FL_MAXSEG ) {
    printf ( "Enter the segment number from 0 - 31\n" );
    scanf ( "%d", &segment );
  }
  printf ( "Please enter a choice: \
1 to erase the block \
2 to fill the segment with 0 \
3 to count from 1 \
4 to enter a string\n" );
  scanf ( "%d", &datamode );
  printf ( "Now pre-reading the segment...   " );
  status = fl_segread ( unit, block, segment );
  printf ( "Preread Status: %d\n", status );
  for ( i = 0; i < FL_SEGSIZE; i++ ) {
    theprintchar = fl_response.payload[i];
    if ( theprintchar < 32 || theprintchar > 126 )
      theprintchar = '.';
    printf ( "%x %c  ", fl_response.payload[i], theprintchar ); // print read data
  }
  printf ( "\nNow writing..." );
  switch ( datamode ) {
     case 1:
       status = fl_blkerase ( unit, block );
       break;
     case 2:
       for ( i = 0; i < FL_SEGSIZE; i++ )
         data[i] = 0;
       status = fl_segwrite ( unit, block, segment, data );
       break;
     case 3:
       for ( i = 0; i < FL_SEGSIZE; i++ )
         data[i] = i;
       status = fl_segwrite ( unit, block, segment, data );
       break;
     case 4:
       printf ( "Please enter  a string less than 16 chars:\n" );
       scanf ( "%s", data );
       status = fl_segwrite ( unit, block, segment, data );
       break;
     default:
       printf ( "No write action taken\n" );
  }
  printf ( "Write Status: %d\n", status );
  printf ( "Now post-reading the segment...   " );
  status = fl_segread ( unit, block, segment );
  printf ( "Postread Status: %d\n", status );
  for ( i = 0; i < FL_SEGSIZE; i++ ) {
    theprintchar = fl_response.payload[i];
    if ( theprintchar < 32 || theprintchar > 126 )
      theprintchar = '.';
    printf ( "%x %c  ", fl_response.payload[i], theprintchar ); // print read data
  }
  printf ( "Thank you for participating in the interactive flash test. \
Have a nice day.\n\n" );
}

void fl_uniqueness_test (  )
{

  // stores random value to the flash and reads it back.
  unsigned char randdata[( FL_MAXBLOCK + 1 ) * ( FL_MAXSEG + 1 ) *
                         FL_SEGSIZE];
  int i, j, k;
  unsigned char unit = 1;
  printf ( "Welcome to flash uniqueness test\n enter 1 to continue\n" );
  scanf ( "%d", &k );
  printf ( "Generating rand data...\n" );
  for ( i = 0; i <= FL_MAXBLOCK; i++ )
    for ( j = 0; j <= FL_MAXSEG; j++ )
      for ( k = 0; k < FL_SEGSIZE; k++ )
        randdata[( i * ( FL_MAXSEG + 1 ) + j ) * FL_SEGSIZE + k] =
          ( unsigned char ) rand (  );
  printf ( "Storing random data...\n" );
  for ( i = 0; i <= FL_MAXBLOCK; i++ ) {
    printf ( "erasing blk %d\n", i );
    if ( fl_blkerase ( unit, i ) == 0 )
      printf ( "erase %d failed\n", i );
    for ( j = 0; j <= FL_MAXSEG; j++ )
      if ( fl_segwrite
           ( unit, i, j,
             randdata + ( ( i * ( FL_MAXSEG + 1 ) + j ) * FL_SEGSIZE ) ) ==
           0 )
        printf ( "write %d %d failed\n", i, j );
  }
  printf ( "Retrieveing random data...\n" );
  for ( i = 0; i <= FL_MAXBLOCK; i++ ) {
    printf ( "reading blk %d \n", i );
    for ( j = 0; j <= FL_MAXSEG; j++ ) {
      if ( fl_segread ( unit, i, j ) == 0 )
        printf ( "read %d %d failed\n", i, j );
      for ( k = 0; k < FL_SEGSIZE; k++ )
        if ( randdata[( i * ( FL_MAXSEG + 1 ) + j ) * FL_SEGSIZE + k] !=
             fl_response.payload[k] )
          printf ( "mismatch %d %d %d\n", i, j, k );
    }
  }
}

void fl_address_assign (  )
{
  unsigned int startloc, endloc, curloc, mode, flag;
  unsigned char data[FL_SEGSIZE] =
    { 1, 3, 2, 4, 3, 5, 4, 6, 5, 7, 6, 8, 7, 2, 1, 7 };

  // GROSS HACK TO HARDCODE SEGSIZE IN INITIALIZATION
  printf ( "Welcome to flash address assign\n\n" );

  while ( 1 ) {
    int input_error = 0;
    printf ( "Please select the mode you wish to program in:\n"
             "\t1 Assign range\n"
             "\t2 Reassign card\n"
             "\t3 Poll id -hook up only 1 card at a time\n"
             "\t4 Clear range\n"
	     "\t5 Set master flag\n"
	     "\t6 Clear master flag\n"
             "\t7 Exit\n" );
    mode = 10;
    while ( !( ( mode >= 1 ) && ( mode <= 7 ) ) ) {
      if ( input_error > 0 )
        printf ( "Invalid selection.  Try again.\n" );
      printf ( "Enter selection number: " );
      if ( scanf ( "%d", &mode ) != 1 ) {
        mode = 10;
        __fpurge ( stdin );
      }
      input_error++;
    }
    switch ( mode ) {
       case ( 1 ):
         flag = 0;
         while ( flag == 0 ) {
           printf ( "Enter starting location (0 returns to main menu)\n" );
           if ( scanf ( "%d", &startloc ) != 1 ) {
             printf ( "Error on input.\n" );
             __fpurge ( stdin );
           } else {
             flag = 1;
           }
         }
         if ( startloc == 0 )
           break;
         flag = 0;
         while ( flag == 0 ) {
           printf ( "Enter ending location\n" );
           if ( scanf ( "%d", &endloc ) != 1 ) {
             printf ( "Error on input.\n" );
             __fpurge ( stdin );
           } else {
             flag = 1;
           }
         }

         for ( curloc = startloc; curloc <= endloc; curloc++ ) {
           flag = 0;
           printf ( "Trying to program location %d...\n", curloc );
           while ( fl_segread ( 255, 0, 0 ) == 0 ) {
             if ( fl_segread ( curloc, 0, 0 ) != 0 ) {  //already have this id
               printf
                 ( "The connected nodewatch network already contains an id %d\n"
                   "Skipping %d.\n", curloc, curloc );
               flag = 1;
               break;
             }
           }
           if ( flag == 1 )
             continue;
           data[0] = curloc;
           /*the following code will write the id to the flash, then probe the
              card every 1/2 second until it either gets a response or 5 seconds
              has elapsed */
           if ( fl_segwrite ( 255, 0, 0, data ) == 0 ) {
             int writecount;
             sleep ( 3 );
             for ( writecount = 0; writecount < 10; writecount++ ) {
               if ( fl_segread ( curloc, 0, 0 ) != 0 )
                 writecount = 100;
               else
                 usleep ( 500000 );
             }
             if ( writecount >= 100 )
               printf ( "write %d succeeded\n\n", curloc );
             else
               printf ( "write %d failed\n\n", curloc );
           } else
             printf ( "   write %d SUCCEEDED!\n", curloc );
         }
         break;



       case ( 2 ):             //reassign range
         flag = 0;
         while ( flag == 0 ) {
           printf ( "Enter id to be reassigned:\n" );
           if ( scanf ( "%d", &startloc ) != 1 ) {
             printf ( "Error on input.\n" );
             __fpurge ( stdin );
           } else {
             flag = 1;
           }
         }
         flag = 0;
         while ( flag == 0 ) {
           printf ( "Enter new id for card:\n" );
           if ( scanf ( "%d", &endloc ) != 1 ) {
             printf ( "Error on input.\n" );
             __fpurge ( stdin );
           } else {
             flag = 1;
           }
         }
	 
         data[0]=endloc; 

	 fl_blkerase(startloc,0);
	 sleep(3);
	 while ( fl_segread (255, 0, 0) == 0 ) {
           sleep(2);
         }
         if ( fl_segwrite ( 255, 0, 0, data ) == 0 ) {
            int writecount;
            sleep ( 3 );
            for ( writecount = 0; writecount < 10; writecount++ ) {
              if ( fl_segread ( endloc, 0, 0 ) != 0 )
                writecount = 100;
              else
                usleep ( 500000 );
            }
            if ( writecount >= 100 )
              printf ( "write succeeded\n\n" );
            else
              printf ( "write failed\n\n" );
         } else {
             printf ( "write succeeded\n\n" );
	 }
		
 
         break;
       case ( 3 ):             //poll id
         flag = 0;
         while ( flag == 0 ) {
           printf ( "Enter starting location (0 returns to main menu)\n" );
           if ( scanf ( "%d", &startloc ) != 1 ) {
             printf ( "Error on input.\n" );
             __fpurge ( stdin );
           } else {
             flag = 1;
           }
         }
         if ( startloc == 0 )
           break;
         flag = 0;
         while ( flag == 0 ) {
           printf ( "Enter ending location\n" );
           if ( scanf ( "%d", &endloc ) != 1 ) {
             printf ( "Error on input.\n" );
             __fpurge ( stdin );
           } else {
             flag = 1;
           }
         }


         curloc = startloc-1;
         while ( 1 ) {
           if ( curloc == endloc )
             curloc = startloc;
           else
             curloc++;
           if ( fl_segread ( curloc, 0, 0 ) != 0 )      //found card
             printf ( "Found id %d\n", curloc );
           else
             printf ( "No card at id %d\n", curloc );
         }
         break;
       case ( 4 ):             //clear range
         printf ( "This mode not yet implemented\n" );
         exit ( 1 );
         break;
       case ( 5 ):		//set master
	 flag = 0;
         while ( flag == 0 ) {
           printf ( "Enter id to be flagged as master:\n" );
           if ( scanf ( "%d", &startloc ) != 1 ) {
             printf ( "Error on input.\n" );
             __fpurge ( stdin );
           } else {
             flag = 1;
           }
         }
         data[0]=startloc;
	 data[2]=0;    //the 3rd byte of the 1st data block is the master flag
         while ( fl_segread(startloc,0,0) == 0 ) sleep(1);
	  
         if ( fl_segwrite(startloc,0,0,data) == 0 ) {
            int writecount;
            sleep ( 3 );
            for ( writecount = 0; writecount < 10; writecount++ ) {
              if ( fl_segread ( startloc, 0, 0 ) != 0 )
                writecount = 100;
              else
                usleep ( 500000 );
            }
            if ( writecount >= 100 )
              printf ( "toggle succeeded\n\n" );
            else
              printf ( "toggle failed\n\n" );
         } else {
             printf ( "toggle succeeded\n\n" );
	 }
         break;
       case ( 6 ):
	 flag = 0;
         while ( flag == 0 ) {
           printf ( "Enter id to have master flag cleared:\n" );
           if ( scanf ( "%d", &startloc ) != 1 ) {
             printf ( "Error on input.\n" );
             __fpurge ( stdin );
           } else {
             flag = 1;
           }
         }
	 
         data[0]=startloc;

	 fl_blkerase(startloc,0);
	 sleep(3);
	 while ( fl_segread (255, 0, 0) == 0 ) {
           sleep(2);
         }
         if ( fl_segwrite ( 255, 0, 0, data ) == 0 ) {
            int writecount;
            sleep ( 3 );
            for ( writecount = 0; writecount < 10; writecount++ ) {
              if ( fl_segread ( startloc, 0, 0 ) != 0 )
                writecount = 100;
              else
                usleep ( 500000 );
            }
            if ( writecount >= 100 )
              printf ( "toggle succeeded\n\n" );
            else
              printf ( "toggle failed\n\n" );
         } else {
             printf ( "toggle succeeded\n\n" );
	 }
		
         break;

       case ( 7 ):             //quit command
         printf ( "Goodbye.\n" );
         exit ( 0 );
         break;
       default:
         printf ( "The code should not have reached this point! BAH!\n" );
         break;
    }
  }
}


void fl_persistence_test (  )
{
}

/// multi-segment or entire flash blocks:

/************ TO DO

int fl_write_big(int length, char *start)  // writes large quantity of data starting at *start; writes backwards; blocks.
// int returns 1 for good and 0 for unsuccesful.
{

}


int fl_read_big(int length, char *start)  // reads large quantity of data into area starting at *start; blocks.
// int returns 1 for good and 0 for unsuccesful.
{

}


///  special override functions:  (toplevel functions for configuration/management modes)
// Note:  these probably should not block.
*********************/
