/* Flash.h  
   Enabling flash storage of configuration parameters

   Michael Jacknis
   Microway

    */

#define DEFAULT_FLASH_OVERRIDE 0        // sets interactive flash debug, etc.
#define FLASH_OVERRIDE_NOTSET 255
// zero is for normal operation with no flash check at the beginning.

// 0 default
// 8 manual flash test
// 9 flash fill test
// 10 set primitive address system.  at 0x4000 it talks to unit
// 256, and writes the new address.  Then it tests the new
// address.

#define FLRESP_CLRBYTE 255      // this byte is used by fl_clearresponse() to clear/invalidate fl_response.

#define TIMER_FLASH 4
#define TIMEOUT_FLASH 2.5       // one second?

#define VALID_BYTE 0xA5         // this must be first in a valid flash block.

#define FL_SETSPEED 0x89        // This is for 18 MHz clock.

// Flash memory map:

#define FL_START_DB 0x4000      // The code ends at 0x16aa.
  // so I think this is a safe limit.

// The following indicates where in flash various data
// is stored, relative to the above FL_START_DB offset.

#define FL_BLK_SIZE 0x200       // 512 byte blocks.

#define FL_MAXBLOCK 27          // The blocks are numbered 0-4... only 5 databases.
// There are 32 blocks in 16384 bytes, but the high few blocks have 
// nasty write protect bits embedded, so let's not venture that far out.

// There are multiple segments in a block.
// Each segment is written or read from the master with a single 
// fixed-size packet.
#define FL_SEGSIZE 16           // 16 data bytes.
#define FL_MAXSEG 31

/*  structure of a generic flash segment:*/
// it's just a character array, so no need for a separate structure.

/* structure of packets used for communicating flash segments: */
typedef struct tagFL_WRREQ_PACKET       // write request packet, also used
// to send back read responses.
{
  unsigned char ucLength;       // always FL_SEGSIZE + 4
  unsigned char ucUnit;         // always us! :)
  unsigned char ucCode;         // always FLASH_WRITE from codes.h
  unsigned char block;          // which 512-byte block (0 = FL_START_DB location)
  unsigned char segment;        // which segment within the block
  unsigned char payload[FL_SEGSIZE];    // the flash data itself, sent as it will be stored.
} FL_WRREQ_PACKET;
typedef struct tagFL_READREQ_PACKET     // read request packet
{
  unsigned char ucLength;
  unsigned char ucUnit;
  unsigned char ucCode;
  unsigned char block;          // which 512-byte block (0 = FL_START_DB location)
  unsigned char segment;        // which segment within the block
} FL_READREQ_PACKET;

// Functions to drive flash management state machine and flash data/ackn receivers:

// Note that whenever flash_override is non-zero, normal queries for measurements are suspended.
// Note also that flash functions can BLOCK, disrupting normal loop in host_l.
void init_flashmachine (  );    // called by host at start.  sets consistent state; starts flash checks if necessary.
void flash_process (  );        // called every time by main host loop.  checks flash_override and flash_state.
// Now that we are using blocking functions, it's handy but less critical/complicated.

// Functions called by execnode to process incoming information:
void fl_copyresponse ( PACKET * inpacket );

/////// Functions to deal with intermediate state machine states:

// NOTE:  the following functions BLOCK, and they call the callback function
// in host_l.c called minimal_net to keep things going.

// The reasoning behind blocking functions:
//  1.  we want each flash storage and read to take place IN ORDER, each with a +ackn.
//  2.  flash storages are not supposed to take a long time.  Disruption should be minimal.
//  3.  flash storages and reads happen infrequently, such as at startup and configuration.
//  4.  The callback functions to keep the network going during the block are simple and easy to implement.
//  5.  the logic and debugging of the flash procedures are clearer.  Programming is simpler and procedural.
//  6.  Despite the future attractiveness, I don't want multithreading at this time for ease/speed of debugging.
void fl_clearresponse (  );     // clears/invalidates recent data
int fl_segread ( char unit, char block, char segment );
int fl_segwrite ( char unit, char block, char segment, unsigned char *ptseg );
int fl_blkerase ( char unit, char block );

void interactive_test (  );
void fl_uniqueness_test (  );
void fl_address_assign (  );
/*  following to be worked on:

/// multi-segment or entire flash blocks:


int fl_write_big(int length, char *start);  // writes large quantity of data starting at *start; writes backwards; blocks.
// int returns 1 for good and 0 for unsuccesful.

int fl_read_big(int length, char *start);  // reads large quantity of data into area starting at *start; blocks.
// int returns 1 for good and 0 for unsuccesful.

///  special override functions:  (toplevel functions for configuration/management modes)
// Note:  these probably should not block.

*/
