// Report node status for Microway cluster
//
// Modification History:
// September something, 2002.. Mike Jacknis added sleep cycles to reset cards.
//    30Jul02 MJNewman: Parse file and emit single node info.
//    15Jul02 MJNewman: Created (as a dummy program)

// standard types

//#include "exec.h"
//#include "cardconfig.h"

#define DISCONNECT_PROBE -10000
#define SHORTED_PROBE -20000

#define MAX_EMAIL_BODY 4095     // nothing should be above a 4K email...
#define EMAIL_INITIAL_BUF 600   // initial allocation for email buffer.

#define INITIALPAUSECOUNTS 10   // first time pause 30 seconds in.
#define PAUSECOUNTS 24000       // pause every 20 hours (72000 seconds)
                          // after that to reset cards.
#define PAUSESLEEPS 15          // Number of seconds to sleep.

#define TIMER_CMDFILE 3
#define TIMEOUT_CMDFILE 3.0     // read file every 3 seconds.
#define TIMEOUT_PAUSE_CMDFILE 30.0      // wait 30 seconds after
                                       // restarting from pause.
#define POWER_ON_STABLE_MEASUREMENT 25  // How long to wait after a power state change
// before reconsidering limits?

// constants used for communicating with the email and shutdown system:


// above is not used because the command file is a FIFO.

#define true (1)
#define false (0)
#define boolean int
#define NUL '\0'
#define STRING_LENGTH 512       // 256


// On Linux this was not defined. On windows version of GCC it was?
#ifndef stricmp
#define stricmp strcasecmp
#endif /*  */

// the following numbers correspond with string constant definitions
// at the beginning of limits.c:
#define LIM_ALERT 0
#define LIM_SHUTDOWN 1
#define LIM_LO  0
#define LIM_HI  1
#define LIM_FAN 0
#define LIM_THERMO 1
#define LIM_VOLT   2

#define POWEROFF_DELAY 60       // this many seconds after command, it should
// definitely be seen as off!
// It was 30, but I was getting errors that it was refusing to cooperate, so made it 60.

/*  LIMITS DATA STRUCTURES BELOW HAVE BEEN REPLACED

// Record the limits on a range of permitted values.
struct limits {
  boolean enabled;
  int min;
  int max;
 };

// Record alerting and shutdown limits
struct typeLimits {
  struct limits alerting;      // show an alert if these are seen
  struct limits shutdown;       // initiate a shutdown if these are seen
};

// Reporting variables for values read
extern struct typeLimits fanLimits;
extern struct typeLimits voltLimits;
extern struct typeLimits tempLimits;

*/

//
//
// Called functions
//

// called once as new section is noticed used to
// call services that close out the previous section
extern void serviceNewSection (  );

// called to parse a line in local context
//
// Inputs:
//      line            line being read
//      lineNumber      used for error reporting
//      fieldName       $xxx from the first thing on the line
//      nodeName        second field on the line
//      value1          third field on the line
extern void serviceFieldLine ( char *line, int lineNumber, int matches,
                               char *fieldName, char *nodeName,
                               char *value1 );

// Parse file containing limit data. Callback functions to parse
// sections as they are read. If no file is found this is quietly
// ignored.
//
// Inputs:
//      inFile          open file to read if NULL try to open inName
//      inName          if the inFile is NULL try to read this one
extern void parseFile ( FILE * inFile, char *inName );
extern void command_process (  );       // looks for new commands in command file.
extern int parseChassisFile ( char *inName );
extern int parseClusterFile ( char *inName );
extern int outNodeFile ( char *outName );


//
// this code is in limits.h

// The goal is to avoid artificial string size limits whereever possible.
// This is accomplished by using dynamic allocation of the strings themselves.
// This also permits the program to not load unneeded data (such as fan labels)

typedef struct tagCHASSIS {
  char *name;                   // label/name for this category of chassis.
  char *fanlabels[MAX_TACK];
  char *templabels[MAX_TEMP];
  char *voltlabels[MAX_VOLT];
  int tackarray[MAX_TACK];     // These reordering arrays are used when accessing the MEASUREMENTS
  int voltarray[MAX_VOLT];     //  They are NOT used when accessing the limit values or the
  int thermoarray[MAX_TEMP];   //  labels.  The labels and limit values are already in printout order.
  int numfans;
  int tempaverage;     // identifies inlet temperature probe, starting with 1.
  int measure_confirm_count;   // minimum number of bad measurements 
  // to permit action

  // (All following new data items require ability to receive from file,
  // temporary variables, and default values.)

  unsigned int measure_confirm_time;    // Minimum amount of time a bad measurement


  // must continously be seen to permit action. (seconds)

  //??? THE FOLLOWING VARIABLES ARE NOT EVEN BEING USED IN THE ALGORITHM!!!:
  unsigned int shutdowncount;
  unsigned int alertcount;
  //??? these variables are not even being used!!!

  unsigned int shutdown_os_time;        // Amount of time to wait after 
  //??? is this actually used correctly?
  // issuing a Linux shutdown, before issuing a poweroff request.

  unsigned int powerswitch_off_time;    // time in seconds required to hold down 
  // power switch to turn off.  This is needed to wait longer for a 
  // poweroff confirmation measurement, after a poweroff command is issued.
  // For now, this information is redundantly hardcoded in the cards.

  unsigned int power_valid_count;       // Minimum number of consecutive 
  // consistent readings to assure a valid power state meaurement
  // related to clusternode->power_supply_state_count 

  unsigned int power_valid_time;        // Minimum time with 
  // consistent readings to assure a valid power state meaurement
  // related to clusternode->power_supply_state_time

  unsigned int power_valid_timeout;     // Maximum time in which to determine
  // power state.


  signed int volt_powerstate_level[MAX_VOLT];   // Special voltage levels
  // for determining power state.  If all voltages are below this level,
  // the power supply is considered to be OFF.


  // The following data types are copied from NODE in exec.h
  // In the future, the data types should be abstracted so they can be changed.

  // Do not use short int because pointers to these values are being passed
  // to generic programs that expect to be working on regular ints.
  // can't just cast the pointer--- that might not change the value 
  // when the pointer is used.


  int voltalertlo[MAX_VOLT];
  int voltalerthi[MAX_VOLT];
  int voltshutdownlo[MAX_VOLT];
  int voltshutdownhi[MAX_VOLT];

  int tempalertlo[MAX_TEMP];
  int tempalerthi[MAX_TEMP];
  int tempshutdownlo[MAX_TEMP];
  int tempshutdownhi[MAX_TEMP];

  int fanalertlo[MAX_TACK];
  int fanalerthi[MAX_TACK];
  int fanshutdownlo[MAX_TACK];
  int fanshutdownhi[MAX_TACK];

  // Now for the fan groups...
  // Fan groups allow sets of redundant fans to be identified
  // such that partial fan failure within the group does not
  // initiate a shutdown.


  int fangroup[MAX_TACK];      // assigns each fan to one group.
  // default would be 0 1 2 3 4 5 6 7 8 9 10 
  int fangroupfail[MAX_TACK];  // max failures for no shutdown.
  // default would be  0 0 0 0 0 0 0 0 0 0 0

  // to use this information, simply keep track of fan group failures, and
  // after examining all the fans, see if there are any excessive failures.

  /* Now for stuff dealing with the computation of whether this chassis is
     ON or OFF:

     We need all this because we need to compute the power state of
     the node before taking any action that might, for example, turn
     off a node that is already off.

     This is slightly tricky because the same physical measurements of
     voltage are being used to deduce both whether the power supply is
     on or off, and also being used to judge whether the power supply
     is functioning properly.

     Also, because of the effects of:
     a) the physical momentum of the fans;
     b) the delay in measuring the fans by counting pulses;
     c) the capacitors in the power supply


   */


  void *nextchassis;            // Pointer to the next chassis; this is a linked list!
  // not made a CHASSIS because it doesn't know what that is yet.  

} CHASSIS;

//
// Following makes up an entry for a node in the cluster linked-list database:

typedef struct tagCLUSTERNODE {
  CHASSIS *chassis;             // Pointer to a chassis with correct chassis info.
  // Null pointer means there is no chassis entry to match!
  // it is not necessary to store chassis category name --
  // this is already stored in the chassis category database!
  unsigned char cardaddress;    // the address on the card
  char *ipaddress;              // the internet address expressed as an ascii string
  int autoalert;                // 1 to automatically 
  int autoshutdown;             // 1 to activate automatic shutdown for this node.
  int serialconsole;            // 1 for has serial console, 0 for doesn't
  int ismaster;                 // 1 for is master, 0 for isn't
  NODE nwdata;                  // actual data as collected by the cards
  // formerly seen as NODE Node[MAXNODES] before the linked list.

  // the following keep track of how many consecutive limits 
  // have been experienced.  A valid reading zeros out the count.
  // a count that exceeds shutdowncount or alertcount.

  // all these have to be cleared to zero just like anything else!


  // note:  if the power turns off, or we turn it off, you want ONE
  // email being sent to that effect, despite that ALL fans would be off etc.

  // also, you only want one email sent for each abnormal situation.

  // so the following keeps track of over/underlimit events,
  // along with what was done about them based on power state changes.
  // (you don't want to turn off a machine that is already off!)

  // Counts is for requiring minimum number of measurements:
  // The following is a count of how many continuous out-of-range
  // measurements have been received.  These values
  // are compared with measure_confirm_count

  int thermo_shutdown_lo_count[MAX_TEMP];
  int thermo_shutdown_hi_count[MAX_TEMP];
  int thermo_alert_lo_count[MAX_TEMP];
  int thermo_alert_hi_count[MAX_TEMP];

  int volt_shutdown_lo_count[MAX_VOLT];
  int volt_shutdown_hi_count[MAX_VOLT];
  int volt_alert_lo_count[MAX_VOLT];
  int volt_alert_hi_count[MAX_VOLT];

  int fan_shutdown_lo_count[MAX_TACK];
  int fan_shutdown_hi_count[MAX_TACK];
  int fan_alert_lo_count[MAX_TACK];
  int fan_alert_hi_count[MAX_TACK];

  // Additionally, for an overlimit condition to trigger,
  // it must remain in that condition for a minimum amount
  // of time.  

  //??? change to integer microseconds.
  // The following times, to be measured in integer microseconds,
  // are compared with measure_confirm_time

  long thermo_shutdown_lo_time[MAX_TEMP];
  long thermo_shutdown_hi_time[MAX_TEMP];
  long thermo_alert_lo_time[MAX_TEMP];
  long thermo_alert_hi_time[MAX_TEMP];

  long volt_shutdown_lo_time[MAX_VOLT];
  long volt_shutdown_hi_time[MAX_VOLT];
  long volt_alert_lo_time[MAX_VOLT];
  long volt_alert_hi_time[MAX_VOLT];

  long fan_shutdown_lo_time[MAX_TACK];
  long fan_shutdown_hi_time[MAX_TACK];
  long fan_alert_lo_time[MAX_TACK];
  long fan_alert_hi_time[MAX_TACK];

  // ... and the analysis of power supply measured state:
  // For this, we need a minimum number of continuously stable
  // measurements spanning a set timeframe, and this has to occur
  // before a timeout.

  char power_supply_state;      // What is the power supply actually doing, this instant?
  int power_supply_state_count; // How many measurements indicated consistent results
  long power_supply_state_time; // ... and for how long.
  long power_supply_state_timeout;      // keep track of timeout.


  // The following maintains a state for every measurement.
  // This allows different measurements to proceed independently 
  // towards being overlimit.  The states are:
  //   Idle, waiting_power, email_pending, email_sent, shutdown_pending

  char thermo_limit_new_state[MAX_TEMP];
  char volt_limit_new_state[MAX_VOLT];
  char fan_limit_new_state[MAX_TACK];

  //  char thermo_limit_recent_state[MAX_TEMP];
  //  char volt_limit_recent_state[MAX_VOLT];
  //  char fan_limit_recent_state[MAX_TACK];

  // NOTE:  The above two sets of states do NOT have the same
  // set of meaning encodings.  See below for state codes for each.

  char power_cmd_state;         // Indicates current power state of node,
  // including if it is on or off, why, and whether a shutdown/poweroff
  // is pending.

  long power_cmd_delay;         // The time, precise to the second,
  // when the lastest power_cmd_state was assumed.
  // Therefore, meaning/usage depends on power_cmd_state.

  pthread_t os_shutdown_thread; // thread identifier of any possible 
  // o/s shutdown effort.

  char os_shutdown_status_flag;
  //??? this, like all others, needs to be INITIALIZED


  // The following notes the last time email including a message
  // about this node was sent.
  // This prevents overloading the administrator with constant reminder email:
  // Of course if there is a new problem, then a new email is sent 
  // right away.

  long last_email_time;

  unsigned long long email_hash;        // stores a bitwise representation of all
  // the problems most recently emailed.


  void *nextnode;               // enables this to be a Linked List!
  // if there is no next node, the pointer is null!
  // CLUSTERNODE was removed.

} CLUSTERNODE;

//
// for power_supply_state:
enum { POWER_SUPPLY_OFF, POWER_SUPPLY_ON };
// this describes the 'instantaneously measured' state

// states for every measurement currently being made:

enum { NEW_LIM_IDLE,
  NEW_LIM_ALERT_LO_WAITING_PWR,
  NEW_LIM_SHUTDOWN_LO_WAITING_PWR,
  NEW_LIM_ALERT_HI_WAITING_PWR,
  NEW_LIM_SHUTDOWN_HI_WAITING_PWR,
  NEW_LIM_ALERT_LO_EMAIL_PENDING,
  NEW_LIM_SHUTDOWN_LO_EMAIL_PENDING,
  NEW_LIM_ALERT_HI_EMAIL_PENDING,
  NEW_LIM_SHUTDOWN_HI_EMAIL_PENDING,
  NEW_LIM_SHUTDOWN_EMAIL_SENT
};                              // unclear if this is redundant.

// states for the most recent stable status known:

enum { RECENT_LIM_OK,
  RECENT_LIM_ALERT_HI,
  RECENT_LIM_ALERT_LO,
  RECENT_LIM_SHUTDOWN_HI,
  RECENT_LIM_SHUTDOWN_LO
};

// for power_cmd_state:

enum { POWER_CMD_IDLE_ON,       // everthing ok, power on.
  //     POWER_CMD_NO_COMM, // no communication with nodewatch board
  POWER_CMD_IDLE_OFF,           // power is off for reasons unknown to nwserver.
  POWER_CMD_SHUTDOWN_OS_TRY,    // A thread has been spun off to 
  // try to send a message to shut down the OS
  // the main loop responds by waiting for the thread
  // to change the state to either ...MSG_SENT or ...MSG_FAIL, below:

  POWER_CMD_SHUTDOWN_OS_MSG_SENT,       // the thread writes this state
  // to indicate successful shutdown message sent.  
  // the thread also writes the time this took place, into
  // power_cmd_state_time to seconds precision.
  // the main loop responds to this signal by waiting for the time
  // to elapse, and powering off.

  POWER_CMD_SHUTDOWN_OS_MSG_FAIL,       // the shutdown thread writes this
  // state to indicate the shutdown message could not be sent
  // to the OS, after several attempts.  Since some time has already
  // elapsed in trying these attempts, and since the OS is having
  // problems as it is, the main loop responds to seeing this
  // signal with an immediate poweroff.

  POWER_CMD_DO_POWEROFF,        // poweroff command sent to nodewatch board
  POWER_CMD_CONFIRM_POWEROFF,   // poweroff command was successful.
  POWER_CMD_POWEROFF_FAIL
};                              // poweroff command was not successful.

// nodewatch board no comm needs to be a separate state,
// or not, it just needs to be sent.
// we just need an email scratchpad space to write emails.  
// if the space should fill up, we send the email and keep going,
// or allocate more memory.

//
// Limits processing, called by execnode.c:   (relocated to bottom)

void checkThermoLimits ( CLUSTERNODE * thechecknode );

void checkVoltLimits ( CLUSTERNODE * thechecknode );

void checkTackLimits ( CLUSTERNODE * thechecknode );


int parseChassisFile ( char *inName );

int parseClusterFile ( char *inName );

int outNodeFile ( char *outName );

void command_process (  );      // called by main loop to check file for command.

void command_open (  );

void timecountcheck ( CLUSTERNODE * thechecknode, int *thecheckcount,
                      long *thechecktime, char *actual_limit_state,
                      char proposed_limit_state );

void start_power_measure ( CLUSTERNODE * thenodechassis );

void checkThermoLimits ( CLUSTERNODE * thechecknode );

void checkVoltLimits ( CLUSTERNODE * thechecknode );

void checkTackLimits ( CLUSTERNODE * thechecknode );

void genLimitsCheck (  );

int email_alloc ( int asksize );

int email_build ( int printedsize );

int email_restrict ( CLUSTERNODE * thenode, unsigned long long newhashvalue );

void email_send (  );

void email_mon (  );

void email_init (  );

void action_limits ( CLUSTERNODE * thenode, char powertruth );

void alert_email_send ( CLUSTERNODE * clustersearch, char alert_shutdn, // is it alert(0) or shutdown(1) msg?
                        // for reference, if a shutdown, the hash ID 
                        // becomes 0, which means always send
                        // (There can't be two shutdowns of the same
                        // node because the node's shutdown state can only
                        // be marked for shutdown once.)
                        char lim_direction,     // is it too low(0) or too high(1)?
                        char measure_type,      // fans(0), thermo(1) or volts(2)?
                        int measure_index,      // index (pre-array) into physical measurement
                        // right now used to calculate email hash ID code number,
                        // which is needed to prevent duplicate email messages.
                        // could/should also be used to retrieve measure_label and values!
                        char *measure_label,    // the actual measurement label
                        int the_measurement,    // actual value before normalizing.
                        int lim_lo_val, // measurement alert lo limit
                        int lim_hi_val );       // measurement alert hi limit

     // formats an email message segment to report an over/underlimit situation.


void auto_shutdown_node ( CLUSTERNODE * thenode );


void shutdown_check ( CLUSTERNODE * thenode );

void chassis_init_more ( CHASSIS * thechassis );

void cluster_init_more ( CLUSTERNODE * thenode );

void cluster_reset_limits ( CLUSTERNODE * thenode );


void send_test_email (  );
boolean isNull ( char *string );
boolean isNumber ( char *string );
void pt_dbug ( int linenmb, char *desc_string, char *the_string, void *val_pointer );
void pt_dbug2 ( int class,int linenmb, char *desc_string, int thevalue, void *val_pointer );

