// Send actions to Microway node control for action.
//
// Modification History:
//    15Jul02 MJNewman: Created (as a dummy program)

#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>

// The name of this program, from the command line used for messages.
const char* program_name;

// standard types
#define true (1)
#define false (0)
#define boolean int
#define defaultcommandfile "/srv/www/htdocs/ganglia/data/nodecommands.dat"

// These flags are gathered from the options and tell us what to do
// later.
static boolean powerCycle = false;
static boolean setKeylock = false;
static boolean keylock = false;
static boolean setPower = false;
static boolean power = true;
static boolean setName = false;
static char *newname = NULL;
static boolean doReset = false;
static char *nodename = NULL;
static char *outFileName = NULL;
static boolean setRS422 = false;
static boolean rs422 = false;
FILE *outFile = NULL;

// Prints usage information for this program to STREAM (typically
// stdout or stderr), and exit the program with EXIT_CODE.  Does not
// return.
//
// Inputs:
//	stream		file to output to
//	exit_code	value to exit with
void print_usage (FILE* stream, int exit_code)
{
    fprintf (stream, "Usage:  %s options nodeID\n", program_name);
    fprintf (stream,
	     "  -c  --cycle            Power off then on\n"
	     "  -h  --help             Display this usage information\n"
	     "  -k  --keyunlock        Unlocks the keyboard on the node\n"
	     "  -K  --keylock	       Locks the keyboard on the node\n"
	     "  -n  --setname newname  Specify new node name for the node\n"
	     "  -o  --output filename  output file name\n"
	     "  -p  --poweroff         Turn off power on the node\n"
	     "  -P  --poweron	       Turn on power on the node\n"
	     "  -r  --reset	       'Press' the reset button on the node\n"
	     "  -G  --rs422on          Turn on serial console\n"
	     "  -g  --rs422off          Turn off serial console\n"
	    );
    exit (exit_code);
}

// Main program entry point.  ARGC contains number of argument list
// elements; ARGV is an array of pointers to them.
int main (int argc, char* argv[])
{
    int next_option;
    int i;

    // A string listing valid short options letters.
    char* const short_options = "chkKn:o:pPrGg+:";
    // An array describing valid long options.
    struct option long_options[] = {
	{ "cycle",     0, NULL, 'c' },
	{ "help",     0, NULL, 'h' },
	{ "keyunlock",     0, NULL, 'k' },
	{ "keylock",     0, NULL, 'K' },
	{ "nodename",     1, NULL, 'n' },
	{ "output",     1, NULL, 'o' },
	{ "poweroff",     0, NULL, 'p' },
	{ "poweron",     0, NULL, 'P' },
	{ "reset",     0, NULL, 'r' },
	{ "rs422on",   0, NULL, 'G' },
	{ "rs422off",  0, NULL, 'g' },
	{ NULL,       0, NULL, 0   }   // Required at end of array.
    };

    // Remember the name of the program, to incorporate in messages.
    program_name = argv[0];

    do {
	next_option = getopt_long (argc, argv, short_options,
				   long_options, NULL);
	switch (next_option)
	{
	    case 'c':
		// -c or --cycle
		powerCycle = true;
		break;

	    case 'h':
		// -h or --help
		// User has requested usage information.  Print it to standard
		// output, and exit with exit code zero (normal termination).
		print_usage (stdout, 0);
		break;
		
	    case 'k':
		// -k or --keyunlock
		setKeylock = true;
		keylock = false;
		break;

	    case 'K':
		// -K or --keylock
		setKeylock = true;
		keylock = true;
		break;

	    case 'n':
		// -n or --setname
		// takes an argument
		setName = true;
		newname = optarg;
		break;

	    case 'o':
		// -o or --output
		// User has requested that output go to this file
		outFileName = optarg;
		break;

	    case 'p':
		// -p or --poweroff
		setPower = true;
		power = false;
		break;

	    case 'P':
		// -P or --poweron
		setPower = true;
		power = true;
		break;

	    case 'r':
		// -r or --reset
		doReset = true;
		break;
      
            case 'G':
	        // -G or --rs422on
		setRS422 = true;
		rs422=true;
		break;

            case 'g':
	        // -g or --rs422off
		setRS422 = true;
		rs422=false;
		break;

	    case '?':
		// The user specified an invalid option.
		// Print usage information to standard error, and exit with exit
		// code one (indicating abonormal termination).
		print_usage (stderr, 1);
		break;

	    case -1:
		// Done with options. 
		break;

	    default:
		// Something else: unexpected.
		abort ();
	};
    } while (next_option != -1);

    // Done with options.  optind points to first non-option argument.
    nodename = argv[optind];
    if (nodename == NULL) {
	fprintf(stderr,"ERROR: A Node name is required!\n\n");
	// Print usage information to standard error, and exit with exit
	// code one (indicating abonormal termination).
	print_usage (stderr, 1);
    };


    for (i = 0;i < 5;i += 1) {
	// Try 5 times for an output file
	if (outFileName == NULL) outFileName=defaultcommandfile;
	outFile = fopen(outFileName,"a");
	if (outFile != NULL) break;
    };
    if (outFile == NULL) {
	if (outFileName != NULL) {
	    fprintf(stderr,"ERROR: can not append to output file %s\n",outFileName);
	} else {
	    fprintf(stderr,"ERROR: No output file\n");
	};
	outFile = stdout;
    };
    printf("On Node: %s\n", nodename);
    if (powerCycle) {
	printf("Power off then on requested for node %s\n",nodename);
	fprintf (outFile,"$VERSION 1\n$DO_POWER_CYCLE %s\n",nodename);
    };
    if (setKeylock) {
	printf("Keyboard lock %s requested for node\n",(keylock) ? "ON" : "OFF",nodename);
	if (keylock) {
	    fprintf (outFile,"$VERSION 1\n$DO_KEY_LOCK %s\n",nodename);
	} else {
	    fprintf (outFile,"$VERSION 1\n$DO_KEY_UNLOCK %s\n",nodename);
	};
    };
    if (setName) {
	printf("Set node name to %s requested for node %s\n",newname,nodename);
	fprintf (outFile,"$VERSION 1\n$DO_SET_NODENAME %s %s\n",nodename,newname);
    };
    if (setPower) {
	printf("Power %s requested for node %s\n",(power) ? "ON" : "OFF",nodename);
	if (power) {
	    fprintf (outFile,"$VERSION 1\n$DO_POWER_ON %s\n",nodename);
	} else {
	    fprintf (outFile,"$VERSION 1\n$DO_POWER_OFF %s\n",nodename);
	};
    };
    if (doReset) {
	printf("Reset node %s requested\n",nodename);
	fprintf (outFile,"$VERSION 1\n$DO_RESET %s\n",nodename);
    };
    if (setRS422) {
        printf("RS422 %s requested for node %s\n", (rs422) ? "ON" :"OFF", nodename);
	if(rs422) {
           fprintf (outFile,"$VERSION 1\n$DO_RS422_ON %s\n",nodename);
        } else {
           fprintf (outFile,"$VERSION 1\n$DO_RS422_OFF %s\n",nodename);
        };
    };
    if (outFile != stdout) {
	fclose(outFile);
	outFile = stdout;
    };
    return 0;
}
